#!/usr/bin/env python3
#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2013-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.

# This script takes as input an optional input kickstart file and an optional
# kickstart syntax version (the latest is assumed, if none specified).  It
# then provides an interactive shell for the user to manipulate the state of
# the input kickstart file and either prints the final results on stdout or to
# a designated output file when the program quits.

# TODO:
# - error reporting always says you are on line 1
# - handle sections like %packages
# - some meta-commands might be nice:
#   - .delete (a previous line)
#   - .help (requires moving help text into each optionparser object?)
#   - .save

# pylint: disable=found-_-in-module-class

import readline
import argparse
import os, six, sys

from pykickstart.i18n import _
from pykickstart.errors import KickstartError, KickstartVersionError
from pykickstart.parser import KickstartParser, preprocessKickstart
from pykickstart.version import DEVEL, makeVersion

##
## INTERNAL COMMANDS
## These are commands that control operation of the kickstart shell and are
## handled by this program.  They are not recognized kickstart commands.
##

class InternalCommand(object):
    def __init__(self):
        self.op = argparse.ArgumentParser()

    def execute(self, parser):
        pass

class ClearCommand(InternalCommand):
    def execute(self, parser):
        version = parser.version
        parser.handler = makeVersion(version)

class QuitCommand(InternalCommand):
    def execute(self, parser):
        raise EOFError

class ShowCommand(InternalCommand):
    def execute(self, parser):
        print(parser.handler)

##
## COMMAND COMPLETION
##

class KickstartCompleter(object):
    def __init__(self, handler, internal):
        # Build up a dict of kickstart commands and their valid options:
        # { command_name: [options] }
        self.commands = {}

        for (cStr, cObj) in handler.commands.items():
            self._add_command(cStr, cObj)

        for (cStr, cObj) in internal.items():
            self._add_command(cStr, cObj)

        self.currentCandidates = []

    def _add_command(self, cStr, cObj):
        self.commands[cStr] = []

        # Internal and simple commands don't have any optparse crud.
        if cStr.startswith(".") or not hasattr(cObj, "op"):
            return

        for opt in cObj.op.option_list:
            self.commands[cStr] += opt._short_opts + opt._long_opts

    def complete(self, _text, state):
        response = None

        # This is the first time Tab has been pressed, so build up a list of matches.
        if state == 0:
            origline = readline.get_line_buffer()
            begin = readline.get_begidx()
            end = readline.get_endidx()

            beingCompleted = origline[begin:end]
            parts = origline.split()

            if not parts:
                # Everything's a match for an empty string.
                self.currentCandidates = sorted(self.commands.keys())
            else:
                try:
                    # Ignore leading whitespace when trying to figure out
                    # completions for a kickstart command.
                    if begin == 0 or origline[:begin].strip() == "":
                        # first word
                        candidates = self.commands.keys()
                    else:
                        # later word
                        candidates = self.commands[parts[0]]

                    if beingCompleted:
                        self.currentCandidates = [w for w in candidates if w.startswith(beingCompleted)]
                    else:
                        self.currentCandidates = list(candidates)
                except (KeyError, IndexError):
                    self.currentCandidates = []

        try:
            response = self.currentCandidates[state]
        except IndexError:
            response = None

        return response

##
## OPTION PROCESSING
##

op = argparse.ArgumentParser()
op.add_argument("-i", "--input", dest="input",
                help=_("a basis file to use for seeding the kickstart data (optional)"))
op.add_argument("-o", "--output", dest="output",
                help=_("the location to write the finished kickstart file, or stdout if not given"))
op.add_argument("-v", "--version", dest="version", default=DEVEL,
                help=_("version of kickstart syntax to validate against"))

opts = op.parse_args(sys.argv[1:])

##
## SETTING UP PYKICKSTART
##

try:
    kshandler = makeVersion(opts.version)
except KickstartVersionError:
    print(_("The version %s is not supported by pykickstart") % opts.version)
    sys.exit(1)

ksparser = KickstartParser(kshandler, followIncludes=True, errorsAreFatal=False)

if opts.input:
    try:
        processedFile = preprocessKickstart(opts.input)
        ksparser.readKickstart(processedFile)
        os.remove(processedFile)
    except KickstartError as e:
        # Errors should just dump you to the prompt anyway.
        print(_("Warning:  The following error occurred when processing the input file:\n%s\n") % e)

internalCommands = {".clear": ClearCommand(),
                    ".show": ShowCommand(),
                    ".quit": QuitCommand()}

##
## SETTING UP READLINE
##

readline.parse_and_bind("tab: complete")
readline.set_completer(KickstartCompleter(kshandler, internalCommands).complete)

# Since everything in kickstart looks like a command line arg, we need to
# remove '-' from the delimiter string.
delims = readline.get_completer_delims()
readline.set_completer_delims(delims.replace('-', ''))

##
## REPL
##

print("Press ^D to exit.")

while True:
    try:
        line = six.moves.input("ks> ")  # pylint: disable=no-member
    except EOFError:
        # ^D was hit, time to quit.
        break
    except KeyboardInterrupt:
        # ^C was hit, time to quit.  Don't be like other programs.
        break

    # All internal commands start with a ., so if that's the beginning of the
    # line, we need to dispatch ourselves.
    if line.startswith("."):
        words = line.split()
        if words[0] in internalCommands:
            try:
                internalCommands[words[0]].execute(ksparser)
            except EOFError:
                # ".quit" was typed, time to quit.
                break
        else:
            print(_("Internal command %s not recognized.") % words[0])

        continue

    # Now process the line of input as if it were a kickstart file - just an
    # extremely short one.
    try:
        ksparser.readKickstartFromString(line)
    except KickstartError as e:
        print(e)

# And finally, print the output kickstart file.
if opts.output:
    with open(opts.output, "w") as fd:
        fd.write(str(ksparser.handler))
else:
    print("\n" + str(ksparser.handler))



import os
import sys
import unittest
import shlex
import glob
import warnings
import re
import six

import importlib

from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning, \
    KickstartParseWarning
from pykickstart.parser import KickstartParser
from pykickstart.version import DEVEL, makeVersion, returnClassForVersion
from pykickstart.i18n import _

class ParserTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self.version = DEVEL

    def setUp(self):
        self._handler = None
        self._parser = None
        unittest.TestCase.setUp(self)

    def tearDown(self):
        """Undo anything performed by setUp"""
        del self._handler
        del self._parser
        self._handler = None
        self._parser = None
        unittest.TestCase.tearDown(self)

    def get_parser(self):
        """This function can be overriden by subclasses,
        for example if the subclass wants to use a fresh
        parser for every test
        """
        if self._parser is None:
            self._parser = KickstartParser(self.handler)
        return self._parser

    @property
    def parser(self):
        return self.get_parser()

    @property
    def handler(self):
        if self._handler is None:
            self._handler = makeVersion(self.version)
        return self._handler

    def assert_parse_error(self, ks_string, exception=KickstartParseError, regex=r".*"):
        """Parsing of this command sequence is expected to raise an exception,
        exception type can be set by the exception keyword argument.

        By default the KickstartParseError is expected.
        """

        with six.assertRaisesRegex(self, exception, regex):
            self.parser.readKickstartFromString(ks_string)

    def assert_parse(self, ks_string):
        """Parsing of his command sequence is expected to finish without
        raising an exception; if it raises an exception, the test failed
        """
        try:
            self.parser.readKickstartFromString(ks_string)
        except Exception as e:      # pylint: disable=broad-except
            self.fail("Failed while parsing commands %s: %s" % (ks_string, e))


class CommandSequenceTest(ParserTest):
    """Kickstart command sequence testing

    Enables testing kickstart indepdent command sequences
    and checking if their parsing raises or doesn't raise
    a parsing exception.
    """

    def setUp(self):
        '''Perform any command setup'''
        ParserTest.setUp(self)

        # turn all parse warnings into errors so we can catch them
        warnings.simplefilter("error", category=KickstartParseWarning)

    def tearDown(self):
        '''Undo anything performed by setUp(self)'''
        # reset warnings
        warnings.resetwarnings()

        ParserTest.tearDown(self)

    def get_parser(self):
        """Command sequence tests need a fresh parser
        for each test"""
        handler = makeVersion(self.version)
        return KickstartParser(handler)


# Base class for any command test case
class CommandTest(unittest.TestCase):
    command = ""

    def setUp(self):
        '''Perform any command setup'''
        unittest.TestCase.setUp(self)

        # turn all parse warnings into errors so we can catch them
        warnings.simplefilter("error", category=KickstartParseWarning)

        # ignore deprecation warnings
        warnings.simplefilter("ignore", category=KickstartDeprecationWarning)

    def tearDown(self):
        '''Undo anything performed by setUp(self)'''
        # reset warnings
        warnings.resetwarnings()

        unittest.TestCase.tearDown(self)

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        self._options = []

    @property
    def handler(self):
        version = self.__class__.__name__.split("_")[0]
        return returnClassForVersion(version)

    @property
    def optionList(self):
        if self._options:
            return self._options

        parser = self.getParser(self.command)._getParser()

        for action in [a for a in parser._get_optional_actions() if not a.deprecated]:
            self._options.extend(action.option_strings)

        return self._options

    def getParser(self, inputStr):
        '''Find a handler using the class name.  Return the requested command
        object.'''
        args = shlex.split(inputStr, comments=True)
        cmd = args[0]

        parser = self.handler().commands[cmd]  # pylint: disable=not-callable
        parser.currentLine = inputStr
        parser.currentCmd = args[0]
        parser.seen = True

        return parser

    def assert_parse(self, inputStr, expectedStr=None):
        '''KickstartParseError is not raised and the resulting string matches
        supplied value'''
        parser = self.getParser(inputStr)
        args = shlex.split(inputStr, comments=True)

        # If expectedStr supplied, we want to ensure the parsed result matches
        if expectedStr is not None:
            obj = parser.parse(args[1:])
            result = str(obj)

            # Strip any comment lines ... we only match on non-comments
            result = re.sub("^#[^\n]*\n", "", result)

            # Ensure we parsed as expected
            self.assertEqual(result, expectedStr)
        # No expectedStr supplied, just make sure it does not raise an
        # exception
        else:
            try:
                obj = parser.parse(args[1:])
            except Exception as e:      # pylint: disable=broad-except
                self.fail("Failed while parsing: %s" % e)
        return obj

    def assert_parse_error(self, inputStr, exception=KickstartParseError, regex=r".*"):
        '''Assert that parsing the supplied string raises a
        KickstartParseError'''
        parser = self.getParser(inputStr)
        args = shlex.split(inputStr, comments=True)

        with six.assertRaisesRegex(self, exception, regex):
            parser.parse(args[1:])

    def assert_deprecated(self, cmd, opt):
        '''Ensure that the provided option is listed as deprecated'''
        parser = self.getParser(cmd)

        for action in parser.op._get_optional_actions():
            if opt in action.option_strings:
                self.assertTrue(action.deprecated)

    def assert_removed(self, cmd, opt):
        '''Ensure that the provided option is not present in option_list'''
        parser = self.getParser(cmd)
        for action in parser.op._get_optional_actions():
            self.assertNotEqual(action.dest, opt)

    def assert_required(self, cmd, opt):
        '''Ensure that the provided option is labelled as required in
        option_list'''
        parser = self.getParser(cmd)
        for action in parser.op._get_optional_actions():
            if opt in action.option_strings:
                self.assertTrue(action.required)

    def assert_type(self, cmd, opt, opt_type):
        '''Ensure that the provided option is of the requested type'''
        parser = self.getParser(cmd)
        for action in parser.op._get_optional_actions():
            if opt in action.option_strings:
                self.assertEqual(action.type.__name__, opt_type)


def loadModules(moduleDir, cls_pattern="_TestCase", skip_list=None):
    '''taken from firstboot/loader.py'''

    if skip_list is None:
        skip_list = ["__init__", "baseclass"]

    # Guaruntee that __init__ is skipped
    if skip_list.count("__init__") == 0:
        skip_list.append("__init__")

    tests = list()

    # Make sure moduleDir is in the system path so imputil works.
    if not moduleDir in sys.path:
        sys.path.insert(0, moduleDir)

    # Get a list of all *.py files in moduleDir
    lst = [os.path.splitext(os.path.basename(x))[0] for x in glob.glob(moduleDir + "/*.py")]

    # Inspect each .py file found
    for module in lst:
        if module in skip_list:
            continue

        # Attempt to load the found module.
        try:
            loaded = importlib.import_module(module)
        except ImportError as e:
            print(_("Error loading module %s: %s") % (module, e))
            continue

        # Find class names that match the supplied pattern (default: "_TestCase")
        beforeCount = len(tests)
        for obj in list(loaded.__dict__.keys()):
            if obj.endswith(cls_pattern):
                tests.append(loaded.__dict__[obj])
        afterCount = len(tests)

        # Warn if no tests found
        if beforeCount == afterCount:
            print(_("Module %s does not contain any test cases; skipping.") % module)
            continue

    return tests



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2006-2015 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Methods for working with kickstart versions.

This module defines several symbolic constants that specify kickstart syntax
versions.  Each version corresponds roughly to one release of Red Hat Linux,
Red Hat Enterprise Linux, or Fedora Core as these are where most syntax
changes take place.

This module also exports several functions:

    makeVersion - Given a version number, return an instance of the
                  matching handler class.

    returnClassForVersion - Given a version number, return the matching
                            handler class.  This does not return an
                            instance of that class, however.

    stringToVersion - Convert a string representation of a version number
                      into the symbolic constant.

    versionToString - Perform the reverse mapping.

    versionToLongString - Perform the reverse mapping but use long names.

    versionFromFile - Read a kickstart file and determine the version of
                      syntax it uses.  This requires the kickstart file to
                      have a version= comment in it.
"""
import os, re, sys

import importlib

from pykickstart.i18n import _

from pykickstart.errors import KickstartVersionError
from pykickstart.load import load_to_str

# Symbolic names for internal version numbers.
RHEL3 = 900
FC3 = 1000
RHEL4 = 1100
FC4 = 2000
FC5 = 3000
FC6 = 4000
RHEL5 = 4100
F7 = 5000
F8 = 6000
F9 = 7000
F10 = 8000
F11 = 9000
F12 = 10000
F13 = 11000
RHEL6 = 11100
F14 = 12000
F15 = 13000
F16 = 14000
F17 = 15000
F18 = 16000
F19 = 17000
F20 = 18000
F21 = 19000
RHEL7 = 19100
F22 = 20000
F23 = 21000
F24 = 22000
F25 = 23000
F26 = 24000
F27 = 25000
F28 = 26000
F29 = 27000
RHEL8 = 27100

# This is for commands backported from newer versions.
F30 = RHEL8
F31 = RHEL8
F32 = RHEL8

# This always points at the latest version and is the default.
DEVEL = F29

# A one-to-one mapping from string representations to version numbers.
versionMap = {
    "DEVEL": DEVEL,
    "FC3": FC3, "FC4": FC4, "FC5": FC5, "FC6": FC6, "F7": F7, "F8": F8,
    "F9": F9, "F10": F10, "F11": F11, "F12": F12, "F13": F13,
    "F14": F14, "F15": F15, "F16": F16, "F17": F17, "F18": F18,
    "F19": F19, "F20": F20, "F21": F21, "F22": F22, "F23": F23,
    "F24": F24, "F25": F25, "F26": F26, "F27": F27, "F28": F28,
    "F29": F29,
    "RHEL3": RHEL3, "RHEL4": RHEL4, "RHEL5": RHEL5, "RHEL6": RHEL6,
    "RHEL7": RHEL7, "RHEL8": RHEL8
}

def stringToVersion(s):
    """Convert string into one of the provided version constants.  Raises
       KickstartVersionError if string does not match anything.
    """
    # First try these short forms.
    try:
        return versionMap[s.upper()]
    except KeyError:
        pass

    # Now try the Fedora versions.
    m = re.match(r"^fedora.* (\d+)$", s, re.I)      # pylint: disable=no-member

    if m and m.group(1):
        if "FC" + m.group(1) in versionMap:
            return versionMap["FC" + m.group(1)]
        elif "F" + m.group(1) in versionMap:
            return versionMap["F" + m.group(1)]
        else:
            raise KickstartVersionError(_("Unsupported version specified: %s") % s)

    # Now try the RHEL versions.
    m = re.match(r"^red hat enterprise linux.* (\d+)([\.\d]*)$", s, re.I)       # pylint: disable=no-member

    if m and m.group(1):
        if "RHEL" + m.group(1) in versionMap:
            return versionMap["RHEL" + m.group(1)]
        else:
            raise KickstartVersionError(_("Unsupported version specified: %s") % s)

    # If nothing else worked, we're out of options.
    raise KickstartVersionError(_("Unsupported version specified: %s") % s)

def versionToString(version, skipDevel=False):
    """Convert version into a string representation of the version number.
       This is the reverse operation of stringToVersion.  Raises
       KickstartVersionError if version does not match anything.
    """
    if not skipDevel and version == versionMap["DEVEL"]:
        return "DEVEL"

    for (key, val) in list(versionMap.items()):
        if key == "DEVEL":
            continue
        elif val == version:
            return key

    raise KickstartVersionError(_("Unsupported version specified: %s") % version)

def versionToLongString(version):
    """
        Convert version into a long string representation.
    """
    result = versionToString(version, True)
    result = result.replace('FC', 'F').replace('F', 'Fedora')
    result = result.replace('RHEL', 'RedHatEnterpriseLinux')
    return result

def versionFromFile(f):
    """Given a file or URL, look for a line starting with #version= and
       return the version number.  If no version is found, return DEVEL.
    """
    v = DEVEL

    contents = load_to_str(f)

    for line in contents.splitlines(True):
        if line.strip() == "":
            continue

        if line[:9] == "#version=":
            v = stringToVersion(line[9:].rstrip())
            break

    return v

def returnClassForVersion(version=DEVEL):
    """Return the class of the syntax handler for version.  version can be
       either a string or the matching constant.  Raises KickstartVersionError
       if version does not match anything.
    """
    try:
        version = int(version)
        module = "%s" % versionToString(version, skipDevel=True)
    except ValueError:
        module = "%s" % version
        version = stringToVersion(version)

    module = module.lower()

    try:
        _path = os.path.join(os.path.dirname(__file__), "handlers/")
        sys.path.extend([_path])
        loaded = importlib.import_module(module)

        for (k, v) in list(loaded.__dict__.items()):
            if k.lower().endswith("%shandler" % module):
                return v
    except:
        raise KickstartVersionError(_("Unsupported version specified: %s") % version)

def makeVersion(version=DEVEL):
    """Return a new instance of the syntax handler for version.  version can be
       either a string or the matching constant.  This function is useful for
       standalone programs which just need to handle a specific version of
       kickstart syntax (as provided by a command line argument, for example)
       and need to instantiate the correct object.
    """
    cl = returnClassForVersion(version)
    return cl()

def getVersionFromCommandClass(cls):
    return versionMap[cls.__name__.split('_')[0]]



import six
import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.logvol import  RHEL5_LogVolData, RHEL6_LogVolData
from pykickstart.commands.logvol import FC3_LogVolData, F9_LogVolData, \
    F12_LogVolData, F17_LogVolData, F18_LogVolData, F20_LogVolData
from pykickstart.version import FC3, RHEL6, F20

if six.PY3:
    requiredError = "arguments are required: %s"
else:
    requiredError = "argument %s is required"

class LogVol_TestCase(unittest.TestCase):
    def runTest(self):
        # new objects are equal
        data1 = FC3_LogVolData()
        data2 = FC3_LogVolData()
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # assert default properties
        self.assertEqual(data1.grow, False)
        self.assertEqual(data1.maxSizeMB, 0)
        self.assertEqual(data1.format, True)
        self.assertEqual(data1.percent, 0)
        self.assertEqual(data1.recommended, False)
        self.assertEqual(data1.preexist, False)

        # test for objects difference
        for atr in ['name', 'vgname']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')

        # more tests for other data members
        self.assertEqual(RHEL5_LogVolData().encrypted, False)
        rhel6_data = RHEL6_LogVolData()
        self.assertEqual(rhel6_data.hibernation, False)
        self.assertEqual(rhel6_data.thin_pool, False)
        self.assertEqual(rhel6_data.thin_volume, False)

        self.assertEqual(F9_LogVolData().encrypted, False)
        self.assertEqual(F12_LogVolData().backuppassphrase, False)
        self.assertEqual(F17_LogVolData().resize, False)
        self.assertEqual(F18_LogVolData().hibernation, False)
        f20_data = F20_LogVolData()
        self.assertEqual(f20_data.thin_pool, False)
        self.assertEqual(f20_data.thin_volume, False)



class FC3_TestCase(CommandTest):
    command = "logvol"

    def __init__(self, *kargs, **kwargs):
        CommandTest.__init__(self, *kargs, **kwargs)
        self.bytesPerInode = ""

    def runTest(self):
        if "--bytes-per-inode" in self.optionList:
            self.bytesPerInode = "--bytes-per-inode=4096 "

        self.assertFalse(self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME") is None)
        self.assertTrue(self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME") !=
                        self.assert_parse("logvol / --size=1024 --name=OTHER --vgname=VGNAME"))
        self.assertFalse(self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME") ==
                         self.assert_parse("logvol / --size=1024 --name=OTHER --vgname=VGNAME"))
        self.assertFalse(self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME") ==
                         self.assert_parse("logvol / --size=1024 --name=NAME --vgname=OTHERVG"))

        # --name and --vgname
        self.assert_parse("logvol / --size=10240 --name=NAME --vgname=VGNAME",
                          "logvol / --size=10240 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --fstype
        self.assert_parse("logvol / --fstype=\"BLAFS\" --size=10240 --name=NAME --vgname=VGNAME",
                          "logvol / --fstype=\"BLAFS\" --size=10240 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --grow
        self.assert_parse("logvol / --grow --size=10240 --name=NAME --vgname=VGNAME",
                          "logvol / --grow --size=10240 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --size
        self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --maxsize
        self.assert_parse("logvol / --maxsize=2048 --size=1024 --name=NAME --vgname=VGNAME",
                          "logvol / --maxsize=2048 --size=1024 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --recommended
        self.assert_parse("logvol / --maxsize=2048 --recommended --size=1024 --name=NAME --vgname=VGNAME",
                          "logvol / --maxsize=2048 --recommended --size=1024 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --percent
        self.assert_parse("logvol / --percent=10 --name=NAME --vgname=VGNAME",
                          "logvol / --percent=10 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --noformat
        # FIXME - should --noformat also be adding --useexisting (seems counter
        # to posted documentation
        # http://fedoraproject.org/wiki/Anaconda/Kickstart)
        self.assert_parse("logvol / --noformat --name=NAME --vgname=VGNAME",
                          "logvol / --noformat --useexisting %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
        # --useexisting
        self.assert_parse("logvol / --useexisting --name=NAME --vgname=VGNAME",
                          "logvol / --useexisting %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

        # assert data types
        self.assert_type("logvol", "size", "int")
        self.assert_type("logvol", "maxsize", "int")
        self.assert_type("logvol", "percent", "int")

        # fail - incorrect type
        self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --size=SIZE", regex="argument --size: invalid int value: 'SIZE'")
        self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --maxsize=MAXSIZE", regex="argument --maxsize: invalid int value: 'MAXSIZE'")
        self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --percent=PCT", regex="argument --percent: invalid int value: 'PCT'")

        # assert required options
        self.assert_required("logvol", "name")
        self.assert_required("logvol", "vgname")

        # fail - missing required
        self.assert_parse_error("logvol / --name=NAME", regex=requiredError % "--vgname")
        self.assert_parse_error("logvol / --vgname=NAME", regex=requiredError % "--name")

        # fail - missing a mountpoint
        if six.PY3:
            self.assert_parse_error("logvol", regex=requiredError % "<mntpoint>, --name, --vgname")
            self.assert_parse_error("logvol --name=NAME", regex=requiredError % "<mntpoint>, --vgname")
            self.assert_parse_error("logvol --vgname=NAME", regex=requiredError % "<mntpoint>, --name")
            self.assert_parse_error("logvol --name=NAME --vgname=NAME", regex=requiredError % "<mntpoint>")
        else:
            self.assert_parse_error("logvol", regex="too few arguments")
            self.assert_parse_error("logvol --name=NAME", regex="too few arguments")
            self.assert_parse_error("logvol --vgname=NAME", regex="too few arguments")
            self.assert_parse_error("logvol --name=NAME --vgname=NAME", regex="too few arguments")

        # fail - unknown options
        self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --bogus-option")

        # extra test coverage
        cmd = self.handler().commands["logvol"]
        cmd.lvList = "--name=blah"
        self.assertEqual(cmd.__str__(), "--name=blah")

class FC3_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC3

    def runTest(self):
        self.assert_parse("""
logvol / --size=1024 --name=nameA --vgname=vgA
logvol /home --size=1024 --name=nameB --vgname=vgA""")

        self.assert_parse("""
logvol / --size=1024 --name=nameA --vgname=vgA
logvol /home --size=1024 --name=nameA --vgname=vgB""")

        self.assert_parse_error("""
logvol / --size=1024 --name=nameA --vgname=vgA
logvol /home --size=1024 --name=nameA --vgname=vgA""", KickstartParseWarning)

class FC4_TestCase(FC3_TestCase):
    def runTest(self):

        # run our baseclass tests first ... but add --bytes-per-inode to each
        # expected result
        FC3_TestCase.runTest(self)

        # --fsoptions
        if "--fsoptions" in self.optionList:
            self.assert_parse("logvol / --fstype=\"BLAFS\" --size=1024 --fsoptions=\"ABC 123\" --name=NAME --vgname=VGNAME",
                              "logvol / --fstype=\"BLAFS\" --size=1024 %s--fsoptions=\"ABC 123\" --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

        if "--bytes-per-inode" in self.optionList:
            # --bytes-per-inode explicit
            self.assert_parse("logvol / --bytes-per-inode=123 --name=NAME --vgname=VGNAME",
                              "logvol / --bytes-per-inode=123 --name=NAME --vgname=VGNAME\n")

            # assert data types
            self.assert_type("logvol", "bytes-per-inode", "int")

            # fail - incorrect type
            self.assert_parse_error("logvol / --bytes-per-inode B --name=NAME --vgname=VGNAME", regex="argument --bytes-per-inode: invalid int value: 'B'")

            # fail - missing value
            self.assert_parse_error("logvol / --bytes-per-inode --name=NAME --vgname=VGNAME", regex="argument --bytes-per-inode: expected one argument")
            self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --bytes-per-inode", regex="argument --bytes-per-inode: expected one argument")

        if "--encrypted" in self.optionList:
            # Just --encrypted
            self.assert_parse("logvol / --size=1024 --encrypted --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--encrypted --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

            # Both --encrypted and --passphrase
            self.assert_parse("logvol / --size=1024 --encrypted --passphrase PASSPHRASE --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--encrypted --passphrase=\"PASSPHRASE\" --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

            # Using --encrypted with --passphrase=<empty>
            self.assert_parse("logvol / --size=1024 --encrypted --passphrase= --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--encrypted --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
            self.assert_parse("logvol / --size=1024 --encrypted --passphrase=\"\" --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--encrypted --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)
            self.assert_parse("logvol / --size=1024 --encrypted --passphrase \"\" --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--encrypted --name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

            # Just --passphrase without --encrypted
            self.assert_parse("logvol / --size=1024 --passphrase=\"PASSPHRASE\" --name=NAME --vgname=VGNAME",
                              "logvol / --size=1024 %s--name=NAME --vgname=VGNAME\n" % self.bytesPerInode)

            # fail - missing value
            self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --encrypted --passphrase", regex="argument --passphrase: expected one argument")

            # fail - --encrypted does not take a value
            self.assert_parse_error("logvol / --encrypted=1 --name=NAME --vgname=VGNAME", regex="argument --encrypted: ignored explicit argument")

class RHEL5_TestCase(FC4_TestCase):
    def runTest(self):
        FC4_TestCase.runTest(self)

class F9_TestCase(FC4_TestCase):
    def runTest(self):
        # Run our baseclass tests first
        FC4_TestCase.runTest(self)

        # assert data types
        self.assert_type("logvol", "fsprofile", "string")

        # fail - missing value
        self.assert_parse_error("logvol / --name=NAME --vgname=VGNAME --fsprofile", regex="argument --fsprofile: expected one argument")

        # Using --fsprofile
        self.assert_parse("logvol / --size=1024 --fsprofile \"FS_PROFILE\" --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --fsprofile=\"FS_PROFILE\" --name=NAME --vgname=VGNAME\n")

        # Ensure --bytes-per-inode has been deprecated
        self.assert_deprecated("logvol", "bytes-per-inode")

class F12_TestCase(F9_TestCase):
    def runTest(self):
        # Run our baseclass tests first
        F9_TestCase.runTest(self)

        # pass
        self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME "
                          "--escrowcert=\"http://x/y\"",
                          "logvol / --size=1024 --name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol / --size=1024 --encrypted --backuppassphrase --name=NAME "
                          "--vgname=VGNAME",
                          "logvol / --size=1024 --encrypted --name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol / --size=1024 --encrypted --escrowcert=\"http://x/y\" "
                          "--name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --encrypted --escrowcert=\"http://x/y\" "
                          "--name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol / --size=1024 --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase --name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol / --size=1024 --encrypted --escrowcert=http://x/y "
                          "--name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --encrypted --escrowcert=\"http://x/y\" "
                          "--name=NAME --vgname=VGNAME\n")

        # fail
        self.assert_parse_error("logvol / --escrowcert --name=NAME --vgname=VGNAME",
                                regex="argument --escrowcert: expected one argument")
        self.assert_parse_error("logvol / --escrowcert --backuppassphrase --name=NAME --vgname=VGNAME",
                                regex="argument --escrowcert: expected one argument")
        self.assert_parse_error("logvol / --encrypted --escrowcert --backuppassphrase --name=NAME --vgname=VGNAME",
                                regex="argument --escrowcert: expected one argument")
        self.assert_parse_error("logvol / --backuppassphrase=False --name=NAME --vgname=VGNAME",
                                regex="argument --backuppassphrase: ignored explicit argument")
        self.assert_parse_error("logvol / --backuppassphrase=True --name=NAME --vgname=VGNAME",
                                regex="argument --backuppassphrase: ignored explicit argument")

class RHEL6_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)

        self.assert_parse("logvol / --size=1024 --fsprofile \"FS_PROFILE\" --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --fsprofile=\"FS_PROFILE\" --name=NAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --encrypted --cipher=3-rot13 --name=NAME --vgname=VGNAME",
                          "logvol / --encrypted --cipher=\"3-rot13\" --name=NAME --vgname=VGNAME\n")
        # Allowed here, but anaconda should complain.  Note how we throw out
        # cipher from the output if there's no --encrypted.
        self.assert_parse("logvol / --cipher=3-rot13 --name=NAME --vgname=VGNAME",
                          "logvol / --name=NAME --vgname=VGNAME\n")

        self.assert_parse_error("logvol / --cipher --name=NAME --vgname=VGNAME", regex="argument --cipher: expected one argument")

        self.assert_parse("logvol swap --hibernation --name=NAME --vgname=VGNAME",
                          "logvol swap --hibernation --name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol swap --recommended --hibernation --name=NAME --vgname=VGNAME",
                          "logvol swap --recommended --hibernation --name=NAME --vgname=VGNAME\n")

        # thinp
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool",
                          "logvol none --thinpool --name=pool1 --vgname=vg\n")
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool "
                          "--chunksize=512",
                          "logvol none --thinpool --chunksize=512 "
                          "--name=pool1 --vgname=vg\n")
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool "
                          "--metadatasize=4 --chunksize=1024",
                          "logvol none --thinpool --metadatasize=4 "
                          "--chunksize=1024 --name=pool1 --vgname=vg\n")
        self.assert_parse("logvol /home --name=home --vgname=vg "
                          "--thin --poolname=pool1",
                          "logvol /home --thin --poolname=pool1 "
                          "--name=home --vgname=vg\n")
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool --profile=performance --size=500",
                          "logvol none --size=500 --thinpool --profile=performance --name=pool1 --vgname=vg\n")

        # missing pool name
        self.assert_parse_error("logvol /home --name=home --vgname=vg --thin")

        # chunksize is an int
        self.assert_parse_error("logvol none --name=pool1 --vgname=vg "
                                "--thinpool --chunksize=foo")

        # both --thin and --thinpool
        self.assert_parse_error("logvol /home --name=home --thin --thinpool --vgname=vg --size=10000")

        # chunksize and/or metadata size and not thinpool
        self.assert_parse_error("logvol none --name=pool1 --vgname=vg "
                                "--chunksize=512")

class RHEL6_AutopartLogVol_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = RHEL6

    def runTest(self):
        # fail - can't use both autopart and logvol
        self.assert_parse_error("""
autopart
logvol / --size=1024 --name=lv --vgname=vg""")

class F14_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)
        self.assert_removed("logvol", "--bytes-per-inode")

class F15_TestCase(F14_TestCase):
    def runTest(self):
        F14_TestCase.runTest(self)
        self.assert_parse("logvol / --size=1024 --name=NAME --vgname=VGNAME --label=ROOT",
                          "logvol / --size=1024 --label=\"ROOT\" --name=NAME --vgname=VGNAME\n")

class F17_TestCase(F15_TestCase):
    def runTest(self):
        F15_TestCase.runTest(self)
        self.assert_parse("logvol /x --name=NAME --size 1000 --vgname=VGNAME "
                          "--useexisting --resize",
                          "logvol /x --size=1000 --useexisting --resize "
                          "--name=NAME --vgname=VGNAME\n")
        self.assert_parse_error("logvol /x --name=NAME --vgname=VGNAME --resize", regex="--resize can only be used in conjunction with --useexisting")

        # no useexisting
        self.assert_parse_error("logvol /x --name=NAME --vgname=VGNAME --resize --size=500", regex="--resize can only be used in conjunction with --useexisting")

        # no size
        self.assert_parse_error("logvol /x --name=NAME --vgname=VGNAME --resize --useexisting", regex="--resize requires --size to indicate new size")

class F18_TestCase(F17_TestCase):
    def runTest(self):
        F17_TestCase.runTest(self)

        self.assert_parse("logvol swap --name=NAME --vgname=VGNAME --hibernation --size=1024",
                          "logvol swap --size=1024 --hibernation --name=NAME --vgname=VGNAME\n")
        self.assert_parse("logvol swap --name=NAME --vgname=VGNAME --recommended --hibernation --size=1024",
                          "logvol swap --recommended --size=1024 --hibernation --name=NAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --size=1024 --encrypted --cipher=3-rot13 --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --encrypted --cipher=\"3-rot13\" --name=NAME --vgname=VGNAME\n")
        # Allowed here, but anaconda should complain.  Note how we throw out
        # cipher from the output if there's no --encrypted.
        self.assert_parse("logvol / --size=1024 --cipher=3-rot13 --name=NAME --vgname=VGNAME",
                          "logvol / --size=1024 --name=NAME --vgname=VGNAME\n")

        self.assert_parse_error("logvol / --cipher --name=NAME --vgname=VGNAME", regex="argument --cipher: expected one argument")

class F20_TestCase(F18_TestCase):
    def runTest(self):
        F18_TestCase.runTest(self)

        self.assert_parse("logvol none --size=1024 --name=pool1 --vgname=vg --thinpool",
                          "logvol none --size=1024 --thinpool --name=pool1 --vgname=vg\n")
        self.assert_parse("logvol none --size=1024 --name=pool1 --vgname=vg "
                          "--thinpool --chunksize=512",
                          "logvol none --size=1024 --thinpool --chunksize=512 "
                          "--name=pool1 --vgname=vg\n")
        self.assert_parse("logvol none --size=1024 --name=pool1 --vgname=vg "
                          "--thinpool --metadatasize=4 --chunksize=1024",
                          "logvol none --size=1024 --thinpool "
                          "--metadatasize=4 --chunksize=1024 --name=pool1 "
                          "--vgname=vg\n")
        self.assert_parse("logvol /home --size=1024 --name=home --vgname=vg "
                          "--thin --poolname=pool1",
                          "logvol /home --size=1024 --thin --poolname=pool1 "
                          "--name=home --vgname=vg\n")

        # missing pool name
        self.assert_parse_error("logvol /home --name=home --vgname=vg --thin",
                                regex="--thin requires --poolname to specify pool name")

        # chunksize is an int
        self.assert_parse_error("logvol none --name=pool1 --vgname=vg "
                                "--thinpool --chunksize=foo",
                                regex="argument --chunksize: invalid int value: 'foo'")

        # both --thin and --thinpool
        self.assert_parse_error("logvol /home --name=home --thin --thinpool --vgname=vg --size=10000", regex="--thin and --thinpool cannot both be specified for the same logvol")

        # chunksize and/or metadata size and not thinpool
        self.assert_parse_error("logvol none --name=pool1 --vgname=vg "
                                "--chunksize=512",
                                regex="--chunksize and --metadatasize are for thin pools only")

        # logvol w/out specified size
        self.assert_parse_error("logvol none --name=pool1 --vgname=vg --thinpool",
                                regex="No size given for logical volume")

        # use existing logvol, which must have a size
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool --useexisting")

        # logvol with a disallowed percent value
        # test with boundary values
        self.assert_parse("logvol / --percent=100 --name=NAME --vgname=VGNAME")
#        self.assert_parse("logvol / --percent=0 --name=NAME --vgname=VGNAME")
        # test with boundary values +/- 1
        self.assert_parse_error("logvol / --percent=101 --name=NAME --vgname=VGNAME",
                                regex="Percentage must be between 0 and 100.")
        self.assert_parse_error("logvol / --percent=-1 --name=NAME --vgname=VGNAME",
                                regex="Percentage must be between 0 and 100.")


class F20_AutopartLogVol_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F20

    def runTest(self):
        # fail - can't use both autopart and logvol
        self.assert_parse_error("""
autopart
logvol / --size=1024 --name=lv --vgname=vg""")

class F21_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        # --profile should work for all logvol commands even though it may be
        # implemented only for some types (thin pool,...)
        self.assert_parse("logvol none --name=pool1 --vgname=vg --thinpool --profile=performance --size=500",
                          "logvol none --size=500 --thinpool --profile=performance --name=pool1 --vgname=vg\n")
        self.assert_parse("logvol /home --name=homelv --vgname=vg --profile=performance --size=500")

        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=2 --percent=30")

class RHEL7_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

        # pass
        self.assert_parse("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing",
                          "logvol / --size=4096 --mkfsoptions=\"some,thing\" --name=LVNAME --vgname=VGNAME\n")

        # can't use --mkfsoptions if you're not formatting
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing --noformat")

        # can't use --mkfsoptions with --fsprofile
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing --fsprofile=PROFILE")

class F23_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

        # can't use --mkfsoptions with --fsprofile
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing --fsprofile=PROFILE")

        # accept cache specifications
        self.assert_parse("logvol /home --name=home --vgname=vg --size=500 --cachesize=250 --cachepvs=pv.01,pv.02 --cachemode=writeback",
                          "logvol /home --size=500 --cachesize=250 --cachepvs=pv.01,pv.02 --cachemode=writeback --name=home --vgname=vg\n")
        # cache mode is not required
        self.assert_parse("logvol /home --name=home --vgname=vg --size=500 --cachesize=250 --cachepvs=pv.01,pv.02")

        # both cache size and cache PVs are required
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachesize=250")
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachepvs=pv.01,pv.02")
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachemode=writeback")
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachesize=250 --cachepvs=pv.01,pv.02 --cachemode=writeback --useexisting")
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachesize=250 --cachepvs=pv.01,pv.02 --cachemode=writeback --noformat")

        # pass
        self.assert_parse("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing",
                          "logvol / --size=4096 --mkfsoptions=\"some,thing\" --name=LVNAME --vgname=VGNAME\n")

        # can't use --mkfsoptions if you're not formatting
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --mkfsoptions=some,thing --noformat")

        # can't use --thin with cache settings
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --cachepvs=pv.01,pv.02 --thin --poolname=POOLNAME")

        # invalid cache mode
        self.assert_parse_error("logvol /home --name=home --vgname=vg --size=500 --cachesize=250 --cachepvs=pv.01,pv.02 --cachemode=bogus")

class F29_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

        self.assert_parse("logvol / --size=4096 --encrypted --luks-version=luks2 --name=LVNAME --vgname=VGNAME",
                          "logvol / --size=4096 --encrypted --luks-version=luks2 --name=LVNAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --size=4096 --encrypted --pbkdf=argon2i --name=LVNAME --vgname=VGNAME",
                          "logvol / --size=4096 --encrypted --pbkdf=argon2i --name=LVNAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --size=4096 --encrypted --pbkdf-memory=256 --name=LVNAME --vgname=VGNAME",
                          "logvol / --size=4096 --encrypted --pbkdf-memory=256 --name=LVNAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --size=4096 --encrypted --pbkdf-time=100 --name=LVNAME --vgname=VGNAME",
                          "logvol / --size=4096 --encrypted --pbkdf-time=100 --name=LVNAME --vgname=VGNAME\n")

        self.assert_parse("logvol / --size=4096 --encrypted --pbkdf-iterations=1000 --name=LVNAME --vgname=VGNAME",
                          "logvol / --size=4096 --encrypted --pbkdf-iterations=1000 --name=LVNAME --vgname=VGNAME\n")

        self.assert_parse_error("logvol / --size=4096 --encrypted --pbkdf-time=100 --pbkdf-iterations=1000 --name=LVNAME --vgname=VGNAME")

class RHEL8_TestCase(F29_TestCase):
    def runTest(self):
        F29_TestCase.runTest(self)
        self.assert_parse_error("logvol / --size=4096 --name=LVNAME --vgname=VGNAME --fstype=btrfs")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
# Peter Jones <pjones@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC6, F10, RHEL6, F17, versionToLongString
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser

class FC6_IscsiData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.ipaddr = kwargs.get("ipaddr", "")
        self.port = kwargs.get("port", 3260)
        self.target = kwargs.get("target", "")
        self.user = kwargs.get("user", None)
        self.password = kwargs.get("password", None)

    def _getArgsAsStr(self):
        retval = ""

        if self.target:
            retval += " --target=%s" % self.target
        if self.ipaddr:
            retval += " --ipaddr=%s" % self.ipaddr
        if self.port != 3260:
            retval += " --port=%s" % self.port
        if self.user:
            retval += " --user=%s" % self.user
        if self.password:
            retval += " --password=%s" % self.password

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "iscsi%s\n" % self._getArgsAsStr()
        return retval

class F10_IscsiData(FC6_IscsiData):
    removedKeywords = FC6_IscsiData.removedKeywords
    removedAttrs = FC6_IscsiData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC6_IscsiData.__init__(self, *args, **kwargs)
        self.user_in = kwargs.get("user_in", None)
        self.password_in = kwargs.get("password_in", None)

    def _getArgsAsStr(self):
        retval = FC6_IscsiData._getArgsAsStr(self)

        if self.user_in:
            retval += " --reverse-user=%s" % self.user_in
        if self.password_in:
            retval += " --reverse-password=%s" % self.password_in

        return retval

class RHEL6_IscsiData(F10_IscsiData):
    removedKeywords = F10_IscsiData.removedKeywords
    removedAttrs = F10_IscsiData.removedAttrs

    def __init__(self, *args, **kwargs):
        F10_IscsiData.__init__(self, *args, **kwargs)
        self.iface = kwargs.get("iface", None)

    def _getArgsAsStr(self):
        retval = F10_IscsiData._getArgsAsStr(self)

        if self.iface:
            retval += " --iface=%s" % self.iface

        return retval

class F17_IscsiData(RHEL6_IscsiData):
    pass

class FC6_Iscsi(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=71, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.iscsi = kwargs.get("iscsi", [])

    def __str__(self):
        retval = ""
        for iscsi in self.iscsi:
            retval += iscsi.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="iscsi", description="""
                            Specifies additional iSCSI storage to be attached
                            during installation. If you use the iscsi parameter,
                            you must also assign a name to the iSCSI node, using
                            the iscsiname parameter. The iscsiname parameter
                            must appear before the iscsi parameter in the
                            kickstart file.

                            We recommend that wherever possible you configure
                            iSCSI storage in the system BIOS or firmware (iBFT
                            for Intel systems) rather than use the iscsi
                            parameter. Anaconda automatically detects and uses
                            disks configured in BIOS or firmware and no special
                            configuration is necessary in the kickstart file.

                            If you must use the iscsi parameter, ensure that
                            networking is activated at the beginning of the
                            installation, and that the iscsi parameter appears
                            in the kickstart file before you refer to iSCSI
                            disks with parameters such as clearpart or
                            ignoredisk.""",
                            version=FC6)
        op.add_argument("--target", help="The target iqn.", version=FC6)
        op.add_argument("--ipaddr", required=True, version=FC6, help="""
                        The IP address of the target to connect to.""")
        op.add_argument("--port", version=FC6, type=int, help="""
                        The port number to connect to (default, --port=3260).
                        """)
        op.add_argument("--user", version=FC6, help="""
                        The username required to authenticate with the target.
                        """)
        op.add_argument("--password", version=FC6, help="""
                        The password that corresponds with the username specified
                        for the target.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        dd = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, dd)
        dd.lineno = self.lineno
        return dd

    def dataList(self):
        return self.iscsi

    @property
    def dataClass(self):
        return self.handler.IscsiData

class F10_Iscsi(FC6_Iscsi):
    removedKeywords = FC6_Iscsi.removedKeywords
    removedAttrs = FC6_Iscsi.removedAttrs

    def _getParser(self):
        op = FC6_Iscsi._getParser(self)
        op.add_argument("--reverse-user", dest="user_in", version=F10, help="""
                        The username required to authenticate with the initiator
                        from a target that uses reverse CHAP authentication.""")
        op.add_argument("--reverse-password", dest="password_in",
                        version=F10, help="""
                        The password that corresponds with the username
                        specified for the initiator.""")
        return op

class RHEL6_Iscsi(F10_Iscsi):
    removedKeywords = F10_Iscsi.removedKeywords
    removedAttrs = F10_Iscsi.removedAttrs

    def _getParser(self):
        op = F10_Iscsi._getParser(self)
        op.add_argument("--iface", version=RHEL6, help="""
                        Bind connection to specific network interface instead
                        of using the default one determined by network layer.
                        Once used, it must be specified for all iscsi commands.
                        """)
        return op

class F17_Iscsi(RHEL6_Iscsi):
    def _getParser(self):
        op = super(F17_Iscsi, self)._getParser()
        for action in op._actions:
            # mark the fact that --iface is available since F17
            # while RHEL6 is based on F12
            if '--iface' in action.option_strings:
                action.help = action.help.replace(
                                versionToLongString(RHEL6),
                                versionToLongString(F17)
                            )

        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2008 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
__all__ = ["F11Handler"]

from pykickstart import commands
from pykickstart.base import BaseHandler
from pykickstart.version import F11

class F11Handler(BaseHandler):
    version = F11

    commandMap = {
        "auth": commands.authconfig.FC3_Authconfig,
        "authconfig": commands.authconfig.FC3_Authconfig,
        "autopart": commands.autopart.F9_AutoPart,
        "autostep": commands.autostep.FC3_AutoStep,
        "bootloader": commands.bootloader.F8_Bootloader,
        "cdrom": commands.cdrom.FC3_Cdrom,
        "clearpart": commands.clearpart.FC3_ClearPart,
        "cmdline": commands.displaymode.FC3_DisplayMode,
        "device": commands.device.F8_Device,
        "deviceprobe": commands.deviceprobe.FC3_DeviceProbe,
        "dmraid": commands.dmraid.FC6_DmRaid,
        "driverdisk": commands.driverdisk.FC4_DriverDisk,
        "firewall": commands.firewall.F10_Firewall,
        "firstboot": commands.firstboot.FC3_Firstboot,
        "graphical": commands.displaymode.FC3_DisplayMode,
        "halt": commands.reboot.FC6_Reboot,
        "harddrive": commands.harddrive.FC3_HardDrive,
        "ignoredisk": commands.ignoredisk.F8_IgnoreDisk,
        "install": commands.upgrade.F11_Upgrade,
        "interactive": commands.interactive.FC3_Interactive,
        "iscsi": commands.iscsi.F10_Iscsi,
        "iscsiname": commands.iscsiname.FC6_IscsiName,
        "keyboard": commands.keyboard.FC3_Keyboard,
        "lang": commands.lang.FC3_Lang,
        "logging": commands.logging.FC6_Logging,
        "logvol": commands.logvol.F9_LogVol,
        "mediacheck": commands.mediacheck.FC4_MediaCheck,
        "method": commands.method.FC6_Method,
        "monitor": commands.monitor.F10_Monitor,
        "multipath": commands.multipath.FC6_MultiPath,
        "network": commands.network.F9_Network,
        "nfs": commands.nfs.FC6_NFS,
        "part": commands.partition.F11_Partition,
        "partition": commands.partition.F11_Partition,
        "poweroff": commands.reboot.FC6_Reboot,
        "raid": commands.raid.F9_Raid,
        "reboot": commands.reboot.FC6_Reboot,
        "repo": commands.repo.F11_Repo,
        "rescue": commands.rescue.F10_Rescue,
        "rootpw": commands.rootpw.F8_RootPw,
        "selinux": commands.selinux.FC3_SELinux,
        "services": commands.services.FC6_Services,
        "shutdown": commands.reboot.FC6_Reboot,
        "skipx": commands.skipx.FC3_SkipX,
        "text": commands.displaymode.FC3_DisplayMode,
        "timezone": commands.timezone.FC6_Timezone,
        "updates": commands.updates.F7_Updates,
        "upgrade": commands.upgrade.F11_Upgrade,
        "url": commands.url.FC3_Url,
        "user": commands.user.F8_User,
        "vnc": commands.vnc.F9_Vnc,
        "volgroup": commands.volgroup.FC3_VolGroup,
        "xconfig": commands.xconfig.F10_XConfig,
        "zerombr": commands.zerombr.F9_ZeroMbr,
        "zfcp": commands.zfcp.FC3_ZFCP,
    }

    dataMap = {
        "DriverDiskData": commands.driverdisk.FC4_DriverDiskData,
        "DeviceData": commands.device.F8_DeviceData,
        "DmRaidData": commands.dmraid.FC6_DmRaidData,
        "IscsiData": commands.iscsi.F10_IscsiData,
        "LogVolData": commands.logvol.F9_LogVolData,
        "MultiPathData": commands.multipath.FC6_MultiPathData,
        "NetworkData": commands.network.F8_NetworkData,
        "PartData": commands.partition.F11_PartData,
        "RaidData": commands.raid.F9_RaidData,
        "RepoData": commands.repo.F11_RepoData,
        "UserData": commands.user.F8_UserData,
        "VolGroupData": commands.volgroup.FC3_VolGroupData,
        "ZFCPData": commands.zfcp.FC3_ZFCPData,
    }



#
# Radek Vykydal <rvykydal@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest


class F28_TestCase(CommandTest):
    command = "nvdimm"

    def runTest(self):
        # pass
        self.assert_parse("nvdimm reconfigure --namespace=namespace0.0 --mode=sector --sectorsize=512",
                          "nvdimm reconfigure --namespace=namespace0.0 --mode=sector --sectorsize=512\n")
        self.assert_parse("nvdimm reconfigure --namespace=whatever --mode=sector --sectorsize=512",
                          "nvdimm reconfigure --namespace=whatever --mode=sector --sectorsize=512\n")

        self.assert_parse("nvdimm use --namespace=whatever",
                          "nvdimm use --namespace=whatever\n")
        self.assert_parse("nvdimm use --blockdevs=pmem0s1",
                          "nvdimm use --blockdevs=pmem0s1\n")
        self.assert_parse("nvdimm use --blockdevs=pmem0s1,pmem0s2",
                          "nvdimm use --blockdevs=pmem0s1,pmem0s2\n")

        # fail
        # --sectorsize is required when recofiguring to sector --mode
        self.assert_parse_error("nvdimm reconfigure --namespace=namespace0.0 --mode=sector")
        self.assert_parse_error("nvdimm reconfigure --namespace=namespace0.0 --mode=invalid --sectorsize=512")
        # --namespace is requried for reconfigure
        self.assert_parse_error("nvdimm reconfigure --mode=sector --sectorsize=512")
        self.assert_parse_error("nvdimm reconfigure extra --namespace=namespace0.0 --mode=sector --sectorsize=512")
        self.assert_parse_error("nvdimm --namespace=namespace0.0 --mode=sector --sectorsize=512")
        self.assert_parse_error("nvdimm invalid_action --namespace=namespace0.0 --mode=sector --sectorsize=512")

        # Only one of --namespace --blockdevs is allowed
        self.assert_parse_error("nvdimm reconfigure --namespace=namespace0.0 --blockdevs=pmem0s1 --mode=sector")
        self.assert_parse_error("nvdimm use --namespace=namespace0.0 --blockdevs=pmem0s1 --mode=sector")
        # Only --namespace is allowed for reconfigure action
        self.assert_parse_error("nvdimm reconfigure --blockdev=pmem0s1 --mode=sector")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

# pylint: disable=found-_-in-module-class

import argparse
import sys
from pykickstart.i18n import _
from pykickstart.base import DeprecatedCommand
from pykickstart.errors import KickstartVersionError
from pykickstart.version import makeVersion, versionMap

def getCommandSet(handler):
    return set(handler.commands.keys())

def getOptSet(lst):
    return set(map(lambda o: o.option_strings[0] if o.option_strings else '', lst))

def printList(lst):
    print(' '.join(lst))

op = argparse.ArgumentParser()
op.add_argument("-f", "--from", dest="f")
op.add_argument("-t", "--to", dest="t")
op.add_argument("-l", "--listversions", dest="listversions", action="store_true",
                default=False,
                help=_("list the available versions of kickstart syntax"))

opts = op.parse_args(sys.argv[1:])

if opts.listversions:
    for key in sorted(versionMap.keys()):
        print(key)

    sys.exit(1)

if not opts.f or not opts.t:
    print(_("You must specify two syntax versions."))
    sys.exit(1)

try:
    fromHandler = makeVersion(opts.f)
except KickstartVersionError:
    print(_("The version %s is not supported by pykickstart") % opts.f)
    sys.exit(1)

try:
    toHandler = makeVersion(opts.t)
except KickstartVersionError:
    print(_("The version %s is not supported by pykickstart") % opts.t)
    sys.exit(1)

fromCmdSet = getCommandSet(fromHandler)
toCmdSet = getCommandSet(toHandler)
bothSet = fromCmdSet & toCmdSet

print(_("The following commands were removed in %s:") % opts.t)
printList(sorted(fromCmdSet - toCmdSet))

print(_("The following commands were deprecated in %s:") % opts.t)
printList(sorted([cmd for cmd in bothSet if isinstance(toHandler.commands[cmd], DeprecatedCommand)]))

print(_("The following commands were added in %s:") % opts.t)
printList(sorted(toCmdSet - fromCmdSet))

print()

for cmd in sorted(bothSet):
    printed = False

    newOptList = []
    deprecatedOptList = []
    removedOptList = []

    fromCmd = fromHandler.commands[cmd]
    toCmd = toHandler.commands[cmd]

    if not hasattr(fromCmd, "op") or not hasattr(toCmd, "op"):
        continue

    fromOpt = fromCmd.op._actions
    toOpt = toCmd.op._actions

    newOptList = getOptSet(toOpt) - getOptSet(fromOpt)
    removedOptList = getOptSet(fromOpt) - getOptSet(toOpt)
    deprecatedOptList = getOptSet([cmd for cmd in toOpt if cmd.deprecated == 1])

    if len(newOptList) > 0:
        print(_("The following options were added to the %(command_name)s command in %(version)s:") % {"command_name": cmd, "version": opts.t})
        printList(sorted(newOptList))
        printed = True

    if len(deprecatedOptList) > 0:
        print(_("The following options were deprecated from the %(command_name)s command in %(version)s:") % {"command_name": cmd, "version": opts.t})
        printList(sorted(deprecatedOptList))
        printed = True

    if len(removedOptList) > 0:
        print(_("The following options were removed from the %(command_name)s command in %(version)s:") % {"command_name": cmd, "version": opts.t})
        printList(sorted(removedOptList))
        printed = True

    if printed:
        print()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC6
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC6_Logging(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.host = kwargs.get("host", "")
        self.level = kwargs.get("level", "")
        self.port = kwargs.get("port", "")

        self._level_provided = bool(self.level)
        if not self._level_provided:
            self.level = "info"

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.level and self._level_provided:
            retval += "# Installation logging level\nlogging --level=%s" % self.level

            if self.host:
                retval += " --host=%s" % self.host

                if self.port:
                    retval += " --port=%s" % self.port

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser(prog="logging", description="""
            This command controls the error logging of anaconda during
            installation. It has no effect on the installed system.""", version=FC6)
        op.add_argument("--host", version=FC6, help="""
            Send logging information to the given remote host, which must be
            running a syslogd process configured to accept remote logging.""")
        op.add_argument("--port", version=FC6, help="""
            If the remote syslogd process uses a port other than the default, it
            may be specified with this option.""")
        op.add_argument("--level", version=FC6, default="info",
                        choices=["debug", "info", "warning", "error", "critical"],
                        help="""
                        Specify the minimum level of messages that appear on tty3.
                        All messages will still be sent to the log file regardless
                        of this level, however.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if self.port and not self.host:
            raise KickstartParseError(_("Can't specify --port without --host."), lineno=self.lineno)

        self._level_provided = True
        return self



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC4, F9, F12, F14, F15, F17, F18, F20, F21, F29
from pykickstart.version import F23, RHEL5, RHEL6, RHEL7, RHEL8, versionToLongString
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser, commaSplit

import warnings
from pykickstart.i18n import _

class FC3_LogVolData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.fstype = kwargs.get("fstype", "")
        self.grow = kwargs.get("grow", False)
        self.maxSizeMB = kwargs.get("maxSizeMB", 0)
        self.name = kwargs.get("name", "")
        self.format = kwargs.get("format", True)
        self.percent = kwargs.get("percent", 0)
        self.recommended = kwargs.get("recommended", False)
        self.size = kwargs.get("size", None)
        self.preexist = kwargs.get("preexist", False)
        self.vgname = kwargs.get("vgname", "")
        self.mountpoint = kwargs.get("mountpoint", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.vgname == y.vgname and self.name == y.name

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""

        if self.fstype:
            retval += " --fstype=\"%s\"" % self.fstype
        if self.grow:
            retval += " --grow"
        if self.maxSizeMB:
            retval += " --maxsize=%d" % self.maxSizeMB
        if not self.format:
            retval += " --noformat"
        if self.percent:
            retval += " --percent=%d" % self.percent
        if self.recommended:
            retval += " --recommended"
        if self.size:
            retval += " --size=%d" % self.size
        if self.preexist:
            retval += " --useexisting"

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)

        args = self._getArgsAsStr()
        args += " --name=%s" % self.name
        args += " --vgname=%s" % self.vgname

        retval += "logvol %s%s\n" % (self.mountpoint, args)
        return retval

class FC4_LogVolData(FC3_LogVolData):
    removedKeywords = FC3_LogVolData.removedKeywords
    removedAttrs = FC3_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC3_LogVolData.__init__(self, *args, **kwargs)
        self.bytesPerInode = kwargs.get("bytesPerInode", 4096)
        self.fsopts = kwargs.get("fsopts", "")

    def _getArgsAsStr(self):
        retval = FC3_LogVolData._getArgsAsStr(self)

        if hasattr(self, "bytesPerInode") and self.bytesPerInode:
            retval += " --bytes-per-inode=%d" % self.bytesPerInode
        if self.fsopts:
            retval += " --fsoptions=\"%s\"" % self.fsopts

        return retval

class RHEL5_LogVolData(FC4_LogVolData):
    removedKeywords = FC4_LogVolData.removedKeywords
    removedAttrs = FC4_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC4_LogVolData.__init__(self, *args, **kwargs)
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

    def _getArgsAsStr(self):
        retval = FC4_LogVolData._getArgsAsStr(self)

        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase:
                retval += " --passphrase=\"%s\"" % self.passphrase

        return retval

class F9_LogVolData(FC4_LogVolData):
    removedKeywords = FC4_LogVolData.removedKeywords + ["bytesPerInode"]
    removedAttrs = FC4_LogVolData.removedAttrs + ["bytesPerInode"]

    def __init__(self, *args, **kwargs):
        FC4_LogVolData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.fsopts = kwargs.get("fsopts", "")
        self.fsprofile = kwargs.get("fsprofile", "")
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

    def _getArgsAsStr(self):
        retval = FC4_LogVolData._getArgsAsStr(self)

        if self.fsprofile:
            retval += " --fsprofile=\"%s\"" % self.fsprofile
        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase:
                retval += " --passphrase=\"%s\"" % self.passphrase

        return retval

class F12_LogVolData(F9_LogVolData):
    removedKeywords = F9_LogVolData.removedKeywords
    removedAttrs = F9_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F9_LogVolData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.escrowcert = kwargs.get("escrowcert", "")
        self.backuppassphrase = kwargs.get("backuppassphrase", False)

    def _getArgsAsStr(self):
        retval = F9_LogVolData._getArgsAsStr(self)

        if self.encrypted and self.escrowcert:
            retval += " --escrowcert=\"%s\"" % self.escrowcert

            if self.backuppassphrase:
                retval += " --backuppassphrase"

        return retval

class RHEL6_LogVolData(F12_LogVolData):
    removedKeywords = F12_LogVolData.removedKeywords
    removedAttrs = F12_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F12_LogVolData.__init__(self, *args, **kwargs)

        self.cipher = kwargs.get("cipher", "")
        self.hibernation = kwargs.get("hibernation", False)

        self.thin_pool = kwargs.get("thin_pool", False)
        self.thin_volume = kwargs.get("thin_volume", False)
        self.pool_name = kwargs.get("pool_name", "")
        self.chunk_size = kwargs.get("chunk_size", None)        # kilobytes
        self.metadata_size = kwargs.get("metadata_size", None)  # megabytes
        self.profile = kwargs.get("profile", "")

    def _getArgsAsStr(self):
        retval = F12_LogVolData._getArgsAsStr(self)

        if self.encrypted and self.cipher:
            retval += " --cipher=\"%s\"" % self.cipher
        if self.hibernation:
            retval += " --hibernation"

        # these are only for thin pools
        if self.thin_pool:
            retval += " --thinpool"

            if self.metadata_size:
                retval += " --metadatasize=%d" % self.metadata_size

            if self.chunk_size:
                retval += " --chunksize=%d" % self.chunk_size

        if self.thin_volume:
            retval += " --thin --poolname=%s" % self.pool_name

        if self.profile:
            retval += " --profile=%s" % self.profile

        return retval

class F14_LogVolData(F12_LogVolData):
    pass

class F15_LogVolData(F14_LogVolData):
    removedKeywords = F14_LogVolData.removedKeywords
    removedAttrs = F14_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F14_LogVolData.__init__(self, *args, **kwargs)
        self.label = kwargs.get("label", "")

    def _getArgsAsStr(self):
        retval = F14_LogVolData._getArgsAsStr(self)

        if self.label:
            retval += " --label=\"%s\"" % self.label

        return retval

class F17_LogVolData(F15_LogVolData):
    removedKeywords = F15_LogVolData.removedKeywords
    removedAttrs = F15_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F15_LogVolData.__init__(self, *args, **kwargs)
        self.resize = kwargs.get("resize", False)

    def _getArgsAsStr(self):
        retval = F15_LogVolData._getArgsAsStr(self)
        if self.resize:
            retval += " --resize"

        return retval

class F18_LogVolData(F17_LogVolData):
    removedKeywords = F17_LogVolData.removedKeywords
    removedAttrs = F17_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F17_LogVolData.__init__(self, *args, **kwargs)
        self.hibernation = kwargs.get("hibernation", False)
        self.cipher = kwargs.get("cipher", "")

    def _getArgsAsStr(self):
        retval = F17_LogVolData._getArgsAsStr(self)

        if self.hibernation:
            retval += " --hibernation"

        if self.encrypted and self.cipher:
            retval += " --cipher=\"%s\"" % self.cipher

        return retval

class F20_LogVolData(F18_LogVolData):
    removedKeywords = F18_LogVolData.removedKeywords
    removedAttrs = F18_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F18_LogVolData.__init__(self, *args, **kwargs)
        self.thin_pool = kwargs.get("thin_pool", False)
        self.thin_volume = kwargs.get("thin_volume", False)
        self.pool_name = kwargs.get("pool_name", "")

        # these are only for thin pools
        self.chunk_size = kwargs.get("chunk_size", None)        # kilobytes
        self.metadata_size = kwargs.get("metadata_size", None)  # megabytes

    def _getArgsAsStr(self):
        retval = F18_LogVolData._getArgsAsStr(self)

        if self.thin_pool:
            retval += " --thinpool"

            if self.metadata_size:
                retval += " --metadatasize=%d" % self.metadata_size

            if self.chunk_size:
                retval += " --chunksize=%d" % self.chunk_size

        if self.thin_volume:
            retval += " --thin --poolname=%s" % self.pool_name

        return retval

class F21_LogVolData(F20_LogVolData):
    removedKeywords = F20_LogVolData.removedKeywords
    removedAttrs = F20_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F20_LogVolData.__init__(self, *args, **kwargs)
        self.profile = kwargs.get("profile", "")

    def _getArgsAsStr(self):
        retval = F20_LogVolData._getArgsAsStr(self)

        if self.profile:
            retval += " --profile=%s" % self.profile

        return retval

class RHEL7_LogVolData(F21_LogVolData):
    removedKeywords = F21_LogVolData.removedKeywords
    removedAttrs = F21_LogVolData.removedAttrs

    def __init__(self, *args, **kwargs):
        F21_LogVolData.__init__(self, *args, **kwargs)
        self.mkfsopts = kwargs.get("mkfsoptions", "") or kwargs.get("mkfsopts", "")

    def _getArgsAsStr(self):
        retval = F21_LogVolData._getArgsAsStr(self)

        if self.mkfsopts:
            retval += " --mkfsoptions=\"%s\"" % self.mkfsopts

        return retval

class F23_LogVolData(F21_LogVolData):
    def __init__(self, *args, **kwargs):
        F21_LogVolData.__init__(self, *args, **kwargs)
        self.cache_size = kwargs.get("cache_size", 0)
        self.cache_mode = kwargs.get("cache_mode", "")
        self.cache_pvs = kwargs.get("cache_pvs", [])
        self.mkfsopts = kwargs.get("mkfsoptions", "") or kwargs.get("mkfsopts", "")

    def _getArgsAsStr(self):
        retval = F21_LogVolData._getArgsAsStr(self)

        if self.cache_size:
            retval += " --cachesize=%d" % self.cache_size
        if self.cache_pvs:
            retval += " --cachepvs=%s" % ",".join(self.cache_pvs)
        if self.cache_mode:
            retval += " --cachemode=%s" % self.cache_mode
        if self.mkfsopts:
            retval += " --mkfsoptions=\"%s\"" % self.mkfsopts

        return retval

class F29_LogVolData(F23_LogVolData):
    def __init__(self, *args, **kwargs):
        F23_LogVolData.__init__(self, *args, **kwargs)
        self.luks_version = kwargs.get("luks_version", "")
        self.pbkdf = kwargs.get("pbkdf", "")
        self.pbkdf_memory = kwargs.get("pbkdf_memory", 0)
        self.pbkdf_time = kwargs.get("pbkdf_time", 0)
        self.pbkdf_iterations = kwargs.get("pbkdf_iterations", 0)

    def _getArgsAsStr(self):
        retval = F23_LogVolData._getArgsAsStr(self)

        if self.encrypted and self.luks_version:
            retval += " --luks-version=%s" % self.luks_version

        if self.encrypted and self.pbkdf:
            retval += " --pbkdf=%s" % self.pbkdf

        if self.encrypted and self.pbkdf_memory:
            retval += " --pbkdf-memory=%s" % self.pbkdf_memory

        if self.encrypted and self.pbkdf_time:
            retval += " --pbkdf-time=%s" % self.pbkdf_time

        if self.encrypted and self.pbkdf_iterations:
            retval += " --pbkdf-iterations=%s" % self.pbkdf_iterations

        return retval

class RHEL8_LogVolData(F29_LogVolData):
    pass

class FC3_LogVol(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=133, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.lvList = kwargs.get("lvList", [])

    def __str__(self):
        retval = ""

        for part in self.lvList:
            retval += part.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="logvol", description="""
                            Create a logical volume for Logical Volume Management
                            (LVM).""", version=FC3, epilog="""
                            Create the partition first, create the logical volume
                            group, and then create the logical volume. For example::

                                part pv.01 --size 3000
                                volgroup myvg pv.01
                                logvol / --vgname=myvg --size=2000 --name=rootvol
                            """)
        op.add_argument("mntpoint", metavar="<mntpoint>", nargs=1, version=FC3,
                        help="Mountpoint for this logical volume or 'none'.")
        op.add_argument("--fstype", version=FC3, help="""
                        Sets the file system type for the logical volume. Valid
                        values include ext4, ext3, ext2, btrfs, swap, and vfat.
                        Other filesystems may be valid depending on command line
                        arguments passed to Anaconda to enable other filesystems.
                        """)
        op.add_argument("--grow", action="store_true", default=False,
                        version=FC3, help="""
                        Tells the logical volume to grow to fill available space
                        (if any), or up to the maximum size setting. Note that
                        ``--grow`` is not supported for logical volumes containing
                        a RAID volume on top of them.""")
        op.add_argument("--maxsize", dest="maxSizeMB", type=int,
                        version=FC3, help="""
                        The maximum size in MiB the logical volume may grow to.
                        Specify an integer value here, and do not append any
                        units.  This option is only relevant if ``--grow`` is
                        specified as well.""")
        op.add_argument("--name", required=True, version=FC3, help="""
                        The name of this logical volume.""")
        op.add_argument("--noformat", action="store_false", version=FC3,
                        dest="format", default=True, help="""
                        Use an existing logical volume and do not format it.
                        """)
        op.add_argument("--percent", dest="percent", type=int,
                        version=FC3, help="""
                        Specify the size of the logical volume as a percentage
                        of available space in the volume group. Without the above
                        ``--grow`` option, this may not work.""")
        op.add_argument("--recommended", action="store_true", default=False,
                        version=FC3, help="""
                        Determine the size of the logical volume automatically.
                        """)
        op.add_argument("--size", type=int, version=FC3, help="""
                        Size of this logical volume.""")
        op.add_argument("--useexisting", dest="preexist", version=FC3,
                        action="store_true", default=False,
                        help="Use an existing logical volume and reformat it.")
        op.add_argument("--vgname", required=True, version=FC3, help="""
                        Name of the Volume Group this logical volume belongs to.
                        """)
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if extra:
            mapping = {"command": "logvol", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        lvd = self.dataClass()  # pylint: disable=not-callable
        self.set_to_obj(ns, lvd)
        lvd.lineno = self.lineno
        lvd.mountpoint = ns.mntpoint[0]

        if not lvd.format:
            lvd.preexist = True

        # Check for duplicates in the data list.
        if lvd in self.dataList():
            warnings.warn(_("A logical volume with the name %(logical_volume_name)s has already been defined in volume group %(volume_group)s.") % {"logical_volume_name": lvd.name, "volume_group": lvd.vgname}, KickstartParseWarning)

        return lvd

    def dataList(self):
        return self.lvList

    @property
    def dataClass(self):
        return self.handler.LogVolData

class FC4_LogVol(FC3_LogVol):
    removedKeywords = FC3_LogVol.removedKeywords
    removedAttrs = FC3_LogVol.removedAttrs

    def _getParser(self):
        op = FC3_LogVol._getParser(self)
        op.add_argument("--bytes-per-inode", dest="bytesPerInode", type=int,
                        version=FC4, help="Specify the bytes/inode ratio.")
        op.add_argument("--fsoptions", dest="fsopts", version=FC4, help="""
                        Specifies a free form string of options to be used when
                        mounting the filesystem. This string will be copied into
                        the ``/etc/fstab`` file of the installed system and should
                        be enclosed in quotes.""")
        return op

class RHEL5_LogVol(FC4_LogVol):
    removedKeywords = FC4_LogVol.removedKeywords
    removedAttrs = FC4_LogVol.removedAttrs

    def _getParser(self):
        op = FC4_LogVol._getParser(self)
        op.add_argument("--encrypted", action="store_true", version=RHEL5,
                        default=False, help="""
                        Specify that this logical volume should be encrypted.
                        """)
        op.add_argument("--passphrase", version=RHEL5, help="""
                        Specify the passphrase to use when encrypting this
                        logical volume. Without the above ``--encrypted``
                        option, this option does nothing. If no passphrase is
                        specified, the default system-wide one is used, or the
                        installer will stop and prompt if there is no default.
                        """)
        return op

class F9_LogVol(FC4_LogVol):
    removedKeywords = FC4_LogVol.removedKeywords
    removedAttrs = FC4_LogVol.removedAttrs

    def _getParser(self):
        op = FC4_LogVol._getParser(self)
        op.add_argument("--bytes-per-inode", deprecated=F9)
        op.add_argument("--fsprofile", version=F9, help="""
                        Specifies a usage type to be passed to the program that
                        makes a filesystem on this partition. A usage type
                        defines a variety of tuning parameters to be used when
                        making a filesystem. For this option to work, the
                        filesystem must support the concept of usage types and
                        there must be a configuration file that lists valid
                        types. For ext2/3/4, this configuration file is
                        ``/etc/mke2fs.conf``.""")
        op.add_argument("--encrypted", action="store_true", default=False,
                        version=F9, help="""
                        Specify that this logical volume should be encrypted.
                        """)
        op.add_argument("--passphrase", version=F9, help="""
                        Specify the passphrase to use when encrypting this
                        logical volume. Without the above ``--encrypted``
                        option, this option does nothing. If no passphrase is
                        specified, the default system-wide one is used, or the
                        installer will stop and prompt if there is no default.
                        """)
        return op

class F12_LogVol(F9_LogVol):
    removedKeywords = F9_LogVol.removedKeywords
    removedAttrs = F9_LogVol.removedAttrs

    def _getParser(self):
        op = F9_LogVol._getParser(self)
        op.add_argument("--escrowcert", metavar="<url>", version=F12, help="""
                        Load an X.509 certificate from ``<url>``. Store the data
                        encryption key of this logical volume, encrypted using
                        the certificate, as a file in ``/root``. Only relevant
                        if ``--encrypted`` is specified as well.""")
        op.add_argument("--backuppassphrase", action="store_true", version=F12,
                        default=False, help="""
                        Only relevant if ``--escrowcert`` is specified as well.
                        In addition to storing the data encryption key, generate
                        a random passphrase and add it to this logical volume.
                        Then store the passphrase, encrypted using the certificate
                        specified by ``--escrowcert``, as a file in ``/root``. If
                        more than one LUKS volume uses ``--backuppassphrase``,
                        the same passphrase will be used for all such volumes.
                        """)
        return op

class RHEL6_LogVol(F12_LogVol):
    removedKeywords = F12_LogVol.removedKeywords
    removedAttrs = F12_LogVol.removedAttrs

    def _getParser(self):
        op = F12_LogVol._getParser(self)
        op.add_argument("--cipher", version=RHEL6, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        op.add_argument("--hibernation", action="store_true", default=False,
                        version=RHEL6, help="""
                        This option can be used to automatically determine the
                        size of the swap partition big enough for hibernation.
                        """)
        op.add_argument("--thinpool", action="store_true", version=RHEL6,
                        dest="thin_pool", default=False, help="""
                        Create a thin pool logical volume. Use a mountpoint of
                        'none'""")
        op.add_argument("--thin", action="store_true", version=RHEL6,
                        dest="thin_volume", default=False, help="""
                        Create a thin logical volume. Requires ``--poolname``.
                        """)
        op.add_argument("--poolname", dest="pool_name", version=RHEL6, help="""
                        Specify the name of the thin pool in which to create a
                        thin logical volume. Requires ``--thin``.""")
        op.add_argument("--chunksize", type=int, dest="chunk_size",
                        version=RHEL6, help="""
                        Specify the chunk size (in KiB) for a new thin pool
                        device.""")
        op.add_argument("--metadatasize", type=int, dest="metadata_size",
                        version=RHEL6, help="""
                        Specify the metadata area size (in MiB) for a new thin
                        pool device.""")
        op.add_argument("--profile", version=RHEL6, help="""
                        Specify an LVM profile for the thin pool (see ``lvm(8)``,
                        standard profiles are ``default`` and ``thin-performance``
                        defined in the ``/etc/lvm/profile/`` directory).""")
        return op

    def parse(self, args):
        # call the overriden method
        retval = F12_LogVol.parse(self, args)
        # the logvol command can't be used together with the autopart command
        # due to the hard to debug behavior their combination introduces
        if self.handler.autopart.seen:
            errorMsg = _("The logvol and autopart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        if retval.thin_volume and retval.thin_pool:
            errorMsg = _("--thin and --thinpool cannot both be specified for "
                         "the same logvol")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        if retval.thin_volume and not retval.pool_name:
            errorMsg = _("--thin requires --poolname to specify pool name")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        if (retval.chunk_size or retval.metadata_size) and \
           not retval.thin_pool:
            errorMsg = _("--chunksize and --metadatasize are for thin pools only")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        return retval

class F14_LogVol(F12_LogVol):
    removedKeywords = F12_LogVol.removedKeywords
    removedAttrs = F12_LogVol.removedAttrs

    def _getParser(self):
        op = F12_LogVol._getParser(self)
        op.remove_argument("--bytes-per-inode", version=F14)
        return op

class F15_LogVol(F14_LogVol):
    removedKeywords = F14_LogVol.removedKeywords
    removedAttrs = F14_LogVol.removedAttrs

    def _getParser(self):
        op = F14_LogVol._getParser(self)
        op.add_argument("--label", version=F15, help="""
                        Specify the label to give to the filesystem to be made.
                        If the given label is already in use by another
                        filesystem, a new label will be created.""")
        return op

class F17_LogVol(F15_LogVol):
    removedKeywords = F15_LogVol.removedKeywords
    removedAttrs = F15_LogVol.removedAttrs

    def _getParser(self):
        op = F15_LogVol._getParser(self)
        op.add_argument("--resize", action="store_true", default=False,
                        version=F17, help="""
                        Attempt to resize this logical volume to the size given
                        by ``--size=``. This option must be used with
                        ``--useexisting --size=``, or an error will be raised.
                        """)
        return op

    def parse(self, args):
        retval = F15_LogVol.parse(self, args)

        if retval.resize and not retval.preexist:
            raise KickstartParseError(_("--resize can only be used in conjunction with --useexisting"), lineno=self.lineno)

        if retval.resize and not retval.size:
            raise KickstartParseError(_("--resize requires --size to indicate new size"), lineno=self.lineno)

        return retval

class F18_LogVol(F17_LogVol):
    removedKeywords = F17_LogVol.removedKeywords
    removedAttrs = F17_LogVol.removedAttrs

    def _getParser(self):
        op = F17_LogVol._getParser(self)
        op.add_argument("--hibernation", action="store_true", default=False,
                        version=F18, help="""
                        This option can be used to automatically determine the
                        size of the swap partition big enough for hibernation.
                        """)

        op.add_argument("--cipher", version=F18, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        return op

class F20_LogVol(F18_LogVol):
    removedKeywords = F18_LogVol.removedKeywords
    removedAttrs = F18_LogVol.removedAttrs

    def _getParser(self):
        op = F18_LogVol._getParser(self)
        op.add_argument("--thinpool", action="store_true", version=F20,
                        dest="thin_pool", default=False, help="""
                        Create a thin pool logical volume. Use a mountpoint
                        of 'none'.""")
        op.add_argument("--thin", action="store_true", version=F20,
                        dest="thin_volume", default=False, help="""
                        Create a thin logical volume. Requires ``--poolname``.
                        """)
        op.add_argument("--poolname", dest="pool_name", version=F20, help="""
                        Specify the name of the thin pool in which to create a
                        thin logical volume. Requires ``--thin``.""")
        op.add_argument("--chunksize", type=int, dest="chunk_size",
                        version=F20, help="""
                        Specify the chunk size (in KiB) for a new thin pool
                        device.""")
        op.add_argument("--metadatasize", type=int, dest="metadata_size",
                        version=F20, help="""
                        Specify the metadata area size (in MiB) for a new thin
                        pool device.""")
        return op

    def parse(self, args):
        retval = F18_LogVol.parse(self, args)

        if retval.thin_volume and retval.thin_pool:
            err = _("--thin and --thinpool cannot both be specified for the same logvol")
            raise KickstartParseError(err, lineno=self.lineno)

        if retval.thin_volume and not retval.pool_name:
            err = _("--thin requires --poolname to specify pool name")
            raise KickstartParseError(err, lineno=self.lineno)

        if (retval.chunk_size or retval.metadata_size) and \
           not retval.thin_pool:
            err = _("--chunksize and --metadatasize are for thin pools only")
            raise KickstartParseError(err, lineno=self.lineno)

        # the logvol command can't be used together with the autopart command
        # due to the hard to debug behavior their combination introduces
        if self.handler.autopart.seen:
            errorMsg = _("The logvol and autopart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        # the same applies to the 'mount' command
        if hasattr(self.handler, "mount") and self.handler.mount.seen:
            errorMsg = _("The logvol and mount commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        if not retval.preexist and not retval.percent and not retval.size and not retval.recommended and not retval.hibernation:
            errorMsg = _("No size given for logical volume. Use one of --useexisting, --noformat, --size, --percent, or --hibernation.")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        if retval.percent is not None and (retval.percent < 0 or retval.percent > 100):
            errorMsg = _("Percentage must be between 0 and 100.")
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        return retval

class F21_LogVol(F20_LogVol):
    removedKeywords = F20_LogVol.removedKeywords
    removedAttrs = F20_LogVol.removedAttrs

    def _getParser(self):
        op = F20_LogVol._getParser(self)
        op.add_argument("--profile", version=F21, help="""
                        Specify an LVM profile for the thin pool (see ``lvm(8)``,
                        standard profiles are ``default`` and ``thin-performance``
                        defined in the ``/etc/lvm/profile/`` directory).""")
        return op

    def parse(self, args):
        retval = F20_LogVol.parse(self, args)

        if retval.size and retval.percent:
            err = _("--size and --percent cannot both be specified for the same logvol")
            raise KickstartParseError(err, lineno=self.lineno)

        return retval

class RHEL7_LogVol(F21_LogVol):
    removedKeywords = F21_LogVol.removedKeywords
    removedAttrs = F21_LogVol.removedAttrs

    def _getParser(self):
        op = F21_LogVol._getParser(self)
        op.add_argument("--mkfsoptions", dest="mkfsopts", version=RHEL7, help="""
                        Specifies additional parameters to be passed to the
                        program that makes a filesystem on this partition. No
                        processing is done on the list of arguments, so they
                        must be supplied in a format that can be passed directly
                        to the mkfs program.  This means multiple options should
                        be comma-separated or surrounded by double quotes,
                        depending on the filesystem.""")
        return op

    def parse(self, args):
        retval = F21_LogVol.parse(self, args)

        if not retval.format and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions with --noformat has no effect."), lineno=self.lineno)

        if retval.fsprofile and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions and --fsprofile cannot be used together."), lineno=self.lineno)

        return retval


class F23_LogVol(F21_LogVol):
    def _getParser(self):
        op = F21_LogVol._getParser(self)
        op.add_argument("--cachesize", type=int, dest="cache_size",
                        version=F23, help="""
                        Requested size (in MiB) of cache attached to the logical
                        volume. Requires ``--cachepvs``.""")
        op.add_argument("--cachemode", dest="cache_mode", version=F23, help="""
                        Mode that should be used for the cache. Either
                        ``writeback`` or ``writethrough``.""")
        op.add_argument("--cachepvs", dest="cache_pvs", type=commaSplit,
                        version=F23, help="""
                        Comma-separated list of (fast) physical volumes that
                        should be used for the cache.""")
        op.add_argument("--mkfsoptions", dest="mkfsopts", version=F23, help="""
                        Specifies additional parameters to be passed to the
                        program that makes a filesystem on this partition. No
                        processing is done on the list of arguments, so they
                        must be supplied in a format that can be passed directly
                        to the mkfs program.  This means multiple options should
                        be comma-separated or surrounded by double quotes,
                        depending on the filesystem.""")
        return op

    def parse(self, args):
        retval = F21_LogVol.parse(self, args)

        if retval.cache_size or retval.cache_mode or retval.cache_pvs:
            if retval.preexist:
                err = _("Adding a cache to an existing logical volume is not supported")
                raise KickstartParseError(err, lineno=self.lineno)

            if retval.thin_volume:
                err = _("Thin volumes cannot be cached")
                raise KickstartParseError(err, lineno=self.lineno)

            if not retval.cache_pvs:
                err = _("Cache needs to have a list of (fast) PVs specified")
                raise KickstartParseError(err, lineno=self.lineno)

            if not retval.cache_size:
                err = _("Cache needs to have size specified")
                raise KickstartParseError(err, lineno=self.lineno)

            if retval.cache_mode and retval.cache_mode not in ("writeback", "writethrough"):
                err = _("Invalid cache mode given: %s") % retval.cache_mode
                raise KickstartParseError(err, lineno=self.lineno)

        if not retval.format and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions with --noformat has no effect."), lineno=self.lineno)

        if retval.fsprofile and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions and --fsprofile cannot be used together."), lineno=self.lineno)

        return retval

class F29_LogVol(F23_LogVol):
    removedKeywords = F23_LogVol.removedKeywords
    removedAttrs = F23_LogVol.removedAttrs

    def _getParser(self):
        op = F23_LogVol._getParser(self)
        op.add_argument("--luks-version", dest="luks_version", version=F29, default="",
                        help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which version of LUKS format should be used to encrypt
                        the filesystem.""")
        op.add_argument("--pbkdf", version=F29, default="", help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        Password-Based Key Derivation Function (PBKDF) algorithm
                        for LUKS keyslot. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-memory", dest="pbkdf_memory", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the memory cost for PBKDF. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-time", dest="pbkdf_time", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of milliseconds to spend with PBKDF passphrase
                        processing. See ``--iter-time`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        op.add_argument("--pbkdf-iterations", dest="pbkdf_iterations", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of iterations directly and avoids PBKDF benchmark.
                        See ``--pbkdf-force-iterations`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        return op

    def parse(self, args):
        retval = F23_LogVol.parse(self, args)

        if retval.pbkdf_time and retval.pbkdf_iterations:
            msg = _("Only one of --pbkdf-time and --pbkdf-iterations can be specified.")
            raise KickstartParseError(msg, lineno=self.lineno)

        return retval

class RHEL8_LogVol(F29_LogVol):
    removedKeywords = F29_LogVol.removedKeywords
    removedAttrs = F29_LogVol.removedAttrs

    def parse(self, args):
        retval = F29_LogVol.parse(self, args)
        if retval.fstype == "btrfs":
            raise KickstartParseError(_("Btrfs file system is not supported"), lineno=self.lineno)
        return retval

    def _getParser(self):
        "Only necessary for the type change documentation"
        op = F29_LogVol._getParser(self)
        for action in op._actions:
            if "--fstype" in action.option_strings:
                action.help += """

                    .. versionchanged:: %s

                    Btrfs support was removed.""" % versionToLongString(RHEL8)
        return op



import re
import os
import tempfile
import unittest
from unittest import TestCase
import unittest.mock as mock
from tools import ksvalidator
from pykickstart import parser
from tests.tools.utils import mktempfile
from pykickstart.version import versionMap
from pykickstart.errors import KickstartDeprecationWarning

class Remove_Non_Existing_File_TestCase(TestCase):
    def runTest(self):
        destdir = tempfile.mkdtemp("", "ksvalidator-test-tmp-", "/tmp")
        # no exception should be raised
        ksvalidator.cleanup(destdir, "/non/existing/file")

class No_Parameters_TestCase(TestCase):
    """
        Executing ksvalidator.py w/o any arguments
        should print usage information.
    """
    def runTest(self):
        retval, messages = ksvalidator.main([])
        self.assertNotEqual(retval, 0)
        self.assertTrue("usage:" in " ".join(messages))

class Show_Param_Help_TestCase(TestCase):
    """
    Help with specified command line options should be printed
    when ksvalidator is run with "--help" argument
    """
    def runTest(self):
        expected_pos_args = {"ksfile"}
        expected_opt_args = {("-h", "--help"),
                             ("-e", "--firsterror"),
                             ("-i", "--followincludes"),
                             ("-l", "--listversions"),
                             ("-v", "--version")}
        retval, messages = ksvalidator.main(["--help"])
        pos_args = set()
        opt_args = set()
        pos = False
        opt = False
        for line in messages:
            if line == "positional arguments:":
                pos = True
                continue
            if line == "optional arguments:":
                pos = False
                opt = True
                continue

            if pos:
                pos_arg = re.search("[a-zA-Z]+", line)
                if pos_arg:
                    pos_args.add(pos_arg.group(0))
            if opt:
                opt_arg = tuple(re.findall("--?[a-zA-Z]+", line))
                if opt_arg:
                    opt_args.add(opt_arg)

        self.assertEqual(expected_pos_args, pos_args)
        self.assertEqual(expected_opt_args, opt_args)
        self.assertEqual(retval, 0)


class KS_With_Errors_TestCase(TestCase):
    def setUp(self):
        super(KS_With_Errors_TestCase, self).setUp()
        ks_content = """
firstb00t --enable
keyboard --vckeymap=cz --xlayoutz='cz'
unknown --foo='bar'
"""
        self._ks_path = mktempfile(ks_content)

    @mock.patch.object(parser, 'print', create=True)
    def runTest(self, _print):
        retval, _out = ksvalidator.main([self._ks_path])
        # kickstart file contains 3 erroneous lines - 3 error messages should be present
        self.assertEqual(_print.call_count, 3)
        self.assertNotEqual(retval, 0)

    def tearDown(self):
        super(KS_With_Errors_TestCase, self).tearDown()
        os.unlink(self._ks_path)


class Stop_On_First_Error_TestCase(TestCase):
    def setUp(self):
        super(Stop_On_First_Error_TestCase, self).setUp()
        ks_content = """
firstb00t --enable
keyboard --vckeymap=cz --xlayoutz='cz'
unknown --foo='bar'
"""
        self._ks_path = mktempfile(ks_content)

    @mock.patch.object(parser, 'print', create=True)
    def runTest(self, _print):
        # kickstart file contains 3 erroneous lines, but ksvalidator is run
        # with "-e" option
        retval, out = ksvalidator.main([self._ks_path, "-e"])
        # print is not called b/c when errorsAreFatal
        # we raise an exception straight away
        self.assertEqual(_print.call_count, 0)
        # there is one error message only, coming from
        # ksvalidator itself
        self.assertTrue("Unknown command: firstb00t" in " ".join(out))
        self.assertNotEqual(retval, 0)

    def tearDown(self):
        super(Stop_On_First_Error_TestCase, self).tearDown()
        os.unlink(self._ks_path)


class KS_With_Include_TestCase(TestCase):
    def setUp(self):
        super(KS_With_Include_TestCase, self).setUp()
        self._include_path = mktempfile("unknown_command --foo=bar", prefix="ks-include")
        ks_content = "autopart --type=lvm\n%%include %s" % self._include_path
        self._ks_path = mktempfile(ks_content)

    @mock.patch.object(parser, 'print', create=True)
    def runTest(self, _print):
        retval, out = ksvalidator.main([self._ks_path])
        self.assertEqual(len(out), 0)  # no output should be present
        self.assertEqual(retval, 0)

        # included snippet has errors
        retval, out = ksvalidator.main(["-i", self._ks_path])
        self.assertEqual(_print.call_count, 1)
        self.assertNotEqual(retval, 0)

    def tearDown(self):
        super(KS_With_Include_TestCase, self).tearDown()
        os.unlink(self._ks_path)
        os.unlink(self._include_path)


class Nonexistent_KS_File_TestCase(TestCase):
    def runTest(self):
        retval, out = ksvalidator.main(["/foo/bar/baz/ks.cfg"])
        self.assertNotEqual(retval, 0)
        self.assertTrue("No such file or directory" in " ".join(out))


@unittest.skipUnless(os.getuid(), "test requires non-root access")
class KS_With_Wrong_Permissions_TestCase(TestCase):
    def setUp(self):
        super(KS_With_Wrong_Permissions_TestCase, self).setUp()
        self._ks_path = mktempfile()
        # create IOError, which further results in KickstartError
        os.chmod(self._ks_path, 0o000)

    def runTest(self):
        retval, out = ksvalidator.main([self._ks_path])
        read_error_seen = False
        for line in out:
            if re.search("Error reading %s.*" % self._ks_path, line):
                read_error_seen = True
        self.assertTrue(read_error_seen)
        self.assertNotEqual(retval, 0)

    def tearDown(self):
        super(KS_With_Wrong_Permissions_TestCase, self).tearDown()
        os.unlink(self._ks_path)


class Wrong_KS_Version_TestCase(TestCase):
    def setUp(self):
        super(Wrong_KS_Version_TestCase, self).setUp()
        self._ks_path = mktempfile()

    def runTest(self):
        # run ksvalidator with nonexistent KS version (FC42)
        retval, out = ksvalidator.main([self._ks_path, "-v", "FC42"])
        self.assertNotEqual(retval, 0)
        self.assertTrue("The version FC42 is not supported by pykickstart" in " ".join(out))

    def tearDown(self):
        super(Wrong_KS_Version_TestCase, self).tearDown()
        os.unlink(self._ks_path)


class List_Versions_TestCase(TestCase):
    def setUp(self):
        self.versions_list = sorted(versionMap.keys())

    def runTest(self):
        retval, out = ksvalidator.main(["-l"])
        self.assertEqual(self.versions_list, out)
        self.assertEqual(retval, 0)

class Raise_KickstartError_TestCase(TestCase):
    def setUp(self):
        super(Raise_KickstartError_TestCase, self).setUp()
        ks_content = "%ksappend /none.ks"
        self._ks_path = mktempfile(ks_content)

    def runTest(self):
        retval, out = ksvalidator.main([self._ks_path, "-v", "F10"])
        self.assertNotEqual(retval, 0)
        self.assertTrue("General kickstart error" in " ".join(out))

    def tearDown(self):
        super(Raise_KickstartError_TestCase, self).tearDown()
        os.unlink(self._ks_path)

class Raise_Exception_TestCase(TestCase):
    def setUp(self):
        super(Raise_Exception_TestCase, self).setUp()
        ks_content = "text'"  # extra quote here
        self._ks_path = mktempfile(ks_content)

    def runTest(self):
        retval, out = ksvalidator.main([self._ks_path, "-v", "F10"])
        self.assertNotEqual(retval, 0)
        self.assertTrue("General error in input file:  No closing quotation" in " ".join(out))

    def tearDown(self):
        super(Raise_Exception_TestCase, self).tearDown()
        os.unlink(self._ks_path)

class Raise_DeprecationWarning_TestCase(TestCase):
    def setUp(self):
        super(Raise_DeprecationWarning_TestCase, self).setUp()
        ks_content = "text"
        self._ks_path = mktempfile(ks_content)

    @mock.patch.object(parser.KickstartParser, 'readKickstart')
    def runTest(self, _mock):
        _mock.side_effect = KickstartDeprecationWarning('Raised by test')
        retval, out = ksvalidator.main([self._ks_path, "-v", "F10"])
        self.assertNotEqual(retval, 0)
        self.assertTrue("File uses a deprecated option or command" in " ".join(out))

    def tearDown(self):
        super(Raise_DeprecationWarning_TestCase, self).tearDown()
        os.unlink(self._ks_path)



# Andy Lindeberg <alindebe@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.rootpw import FC3_RootPw, F8_RootPw

class FC3_TestCase(CommandTest):
    command = "rootpw"

    def runTest(self):
        # assert default values
        self.assertFalse(FC3_RootPw().isCrypted)

        # pass
        self.assert_parse("rootpw --iscrypted secrethandshake", "rootpw --iscrypted secrethandshake\n")

        # fail
        self.assert_parse_error("rootpw")
        self.assert_parse_error("rootpw --iscrypted=OMGSEKRITZ")
        self.assert_parse_error("rootpw --iscrypted")
        self.assert_parse_error("rootpw pwd1 pwd2")

        # fail - unknown option
        self.assert_parse_error("rootpw pass-phrase --bogus-option")
        self.assert_parse_error("rootpw --bogus-option")

class F8_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # assert default values
        self.assertFalse(F8_RootPw().lock)

        # pass
        self.assert_parse("rootpw --lock secrethandshake", "rootpw --lock --plaintext secrethandshake\n")
        self.assert_parse("rootpw --plaintext secrethandshake", "rootpw --plaintext secrethandshake\n")
        self.assert_parse("rootpw --plaintext --iscrypted secrethandshake", "rootpw --iscrypted secrethandshake\n")
        self.assert_parse("rootpw --iscrypted --plaintext secrethandshake\n", "rootpw --plaintext secrethandshake\n")
        self.assert_parse("rootpw --lock --plaintext secrethandshake", "rootpw --lock --plaintext secrethandshake\n")
        self.assert_parse("rootpw --iscrypted --lock secrethandshake", "rootpw --iscrypted --lock secrethandshake\n")
        self.assert_parse("rootpw --lock --iscrypted --plaintext secrethandshake", "rootpw --lock --plaintext secrethandshake\n")
        self.assert_parse("rootpw --lock --plaintext --iscrypted secrethandshake", "rootpw --iscrypted --lock secrethandshake\n")
        self.assert_parse("rootpw --plaintext --iscrypted --lock secrethandshake", "rootpw --iscrypted --lock secrethandshake\n")
        self.assert_parse("rootpw --iscrypted --plaintext --lock secrethandshake", "rootpw --lock --plaintext secrethandshake\n")
        obj = self.assert_parse("rootpw --plaintext \"comment#inpassword\"", "rootpw --plaintext \"comment#inpassword\"\n")
        self.assertEqual(obj.password, "comment#inpassword")

        # fail
        self.assert_parse_error("rootpw --plaintext=ISEEENGLAND secrethandshake")
        self.assert_parse_error("rootpw --lock=NOKEYSFORYOU secrethandshake")
        self.assert_parse_error("rootpw --plaintext")

        if self.__class__.__name__ == "F8_TestCase":
            self.assert_parse_error("rootpw --lock")

class F18_TestCase(F8_TestCase):
    def runTest(self):
        F8_TestCase.runTest(self)

        self.assert_parse("rootpw --lock", "rootpw --lock\n")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent

from pykickstart.errors import KickstartParseWarning, KickstartParseError
from pykickstart.version import FC3, F12, F14, RHEL8, versionToLongString
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class FC3_ZFCPData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.devnum = kwargs.get("devnum", "")
        self.wwpn = kwargs.get("wwpn", "")
        self.fcplun = kwargs.get("fcplun", "")
        self.scsiid = kwargs.get("scsiid", "")
        self.scsilun = kwargs.get("scsilun", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.devnum == y.devnum and self.wwpn == y.wwpn and \
               self.fcplun == y.fcplun and self.scsiid == y.scsiid and \
               self.scsilun == y.scsilun

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "zfcp"

        if self.devnum:
            retval += " --devnum=%s" % self.devnum
        if self.wwpn:
            retval += " --wwpn=%s" % self.wwpn
        if self.fcplun:
            retval += " --fcplun=%s" % self.fcplun
        if hasattr(self, "scsiid") and self.scsiid:
            retval += " --scsiid=%s" % self.scsiid
        if hasattr(self, "scsilun") and self.scsilun:
            retval += " --scsilun=%s" % self.scsilun

        return retval + "\n"

class F12_ZFCPData(FC3_ZFCPData):
    removedKeywords = FC3_ZFCPData.removedKeywords + ["scsiid", "scsilun"]
    removedAttrs = FC3_ZFCPData.removedAttrs + ["scsiid", "scsilun"]

    def __init__(self, *args, **kwargs):
        FC3_ZFCPData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

    def __eq__(self, y):
        if not y:
            return False

        return self.devnum == y.devnum and self.wwpn == y.wwpn and \
               self.fcplun == y.fcplun

class F14_ZFCPData(F12_ZFCPData):
    pass

class RHEL8_ZFCPData(F14_ZFCPData):
    pass

class FC3_ZFCP(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=71, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.zfcp = kwargs.get("zfcp", [])

    def __str__(self):
        retval = ""
        for zfcp in self.zfcp:
            retval += zfcp.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="zfcp", version=FC3, description="""
                        Define a Fibre channel device. This option only applies
                        on IBM System z.""")
        op.add_argument("--devnum", required=True, version=FC3, help="""
                        The device number (zFCP adaptor device bus ID).""")
        op.add_argument("--fcplun", required=True, version=FC3, help="""
                        The device's Logical Unit Number (LUN). Takes the form
                        of a 16-digit number, preceded by 0x.""")
        op.add_argument("--wwpn", required=True, version=FC3, help="""
                        The device's World Wide Port Name (WWPN). Takes the form
                        of a 16-digit number, preceded by 0x.""")
        op.add_argument("--scsiid", required=True, version=FC3, help="SCSI ID")
        op.add_argument("--scsilun", required=True, version=FC3, help="SCSI LUN")
        return op

    def parse(self, args):
        zd = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_obj(ns, zd)
        zd.lineno = self.lineno

        # Check for duplicates in the data list.
        if zd in self.dataList():
            warnings.warn(_("A zfcp with this information has already been defined."), KickstartParseWarning)

        return zd

    def dataList(self):
        return self.zfcp

    @property
    def dataClass(self):
        return self.handler.ZFCPData

class F12_ZFCP(FC3_ZFCP):
    removedKeywords = FC3_ZFCP.removedKeywords
    removedAttrs = FC3_ZFCP.removedAttrs + ["scsiid", "scsilun"]

    def __init__(self, *args, **kwargs):
        FC3_ZFCP.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

    def _getParser(self):
        op = FC3_ZFCP._getParser(self)
        op.add_argument("--scsiid", deprecated=F12)
        op.add_argument("--scsilun", deprecated=F12)
        return op

class F14_ZFCP(F12_ZFCP):
    removedKeywords = F12_ZFCP.removedKeywords
    removedAttrs = F12_ZFCP.removedAttrs

    def _getParser(self):
        op = F12_ZFCP._getParser(self)
        op.remove_argument("--scsiid", version=F14)
        op.remove_argument("--scsilun", version=F14)
        return op

class RHEL8_ZFCP(F14_ZFCP):
    removedKeywords = F14_ZFCP.removedKeywords
    removedAttrs = F14_ZFCP.removedAttrs

    def _getParser(self):
        op = F14_ZFCP._getParser(self)
        op.description += dedent("""

            .. versionchanged:: %s

            It is sufficient to specify an FCP device bus ID if automatic LUN scanning
            is available. Otherwise all three parameters are required.

            ``zfcp --devnum=<devnum> [--wwpn=<wwpn> --fcplun=<lun>]``

            Automatic LUN scanning is available for FCP devices operating in NPIV mode
            if it is not disabled through the `zfcp.allow_lun_scan` module parameter
            (enabled by default). It provides access to all SCSI devices, that is, WWPNs
            and FCP LUNs, found in the storage area network attached to the FCP device
            with the specified bus ID.

        """ % versionToLongString(RHEL8))

        op.epilog += dedent("""
        For example::

            zfcp --devnum=0.0.6000
            zfcp --devnum=0.0.4000 --wwpn=0x5005076300C213e9 --fcplun=0x5022000000000000

        """)

        op.add_argument("--wwpn", default="", required=False, version=RHEL8, help="""
                        The argument is optional.""")
        op.add_argument("--fcplun", default="", required=False, version=RHEL8, help="""
                        The argument is optional.""")
        return op

    def parse(self, args):
        data = F14_ZFCP.parse(self, args)

        if not ((data.devnum and not data.wwpn and not data.fcplun)
                or (data.devnum and data.wwpn and data.fcplun)):
            msg = _("Only --devnum or --devnum with --wwpn and --fcplun are allowed.")
            raise KickstartParseError(msg, lineno=self.lineno)

        return data
